/**

    Copyright 2004, 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.cms;

import java.util.List;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.cms.exceptions.OperationExecutionException;
import it.eng.spago.cms.operations.DeleteOperation;
import it.eng.spago.cms.operations.ExportOperation;
import it.eng.spago.cms.operations.GetOperation;
import it.eng.spago.cms.operations.ImportOperation;
import it.eng.spago.cms.operations.RestoreOperation;
import it.eng.spago.cms.operations.SearchOperation;
import it.eng.spago.cms.operations.SetOperation;
import it.eng.spago.cms.operations.VersionOperation;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;

/**
 * Facade object useful to call operations on the repository
 */
public class CmsManager {

	/**
	 * Instace of an implementation of the IExecutionService interface
	 */
	IExecutionService execService = null;
	
	/**
	 * Constructor of class, reads the the configuration file, gets the class name of
	 * the IExecutionService interface and create an instance of the class. One of the 
	 * Spago configuration file must contain an xml structure like this:
	 * <CONTENTCONFIGURATION>
	 * 		<EXECUTIONSERVICE class="complete name of the class" />
	 */
	public CmsManager() {
		try {
			ConfigSingleton config = ConfigSingleton.getInstance();
			SourceBean execServiceSB = (SourceBean)config.getAttribute("CONTENTCONFIGURATION.EXECUTIONSERVICE");
			String classNameExecService = (String)execServiceSB.getAttribute("class");
			Class classExecService = Class.forName(classNameExecService);
			execService = (IExecutionService)classExecService.newInstance();
		} catch(Exception e) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                    			"CmsManager::init: errore during execution service creation "+e);
		}
 	}
	
	/**
	 * Execs a Set operation using a default user credentials
	 * @param setOp Configuration of the set operation
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public void execSetOperation(SetOperation setOp) throws OperationExecutionException {
		if(!controlService())
			return;
		execService.doSetOperation(setOp) ;
	}
	
	/**
	 * Execs a Set operation using a specific user name and password
	 * @param setOp Configuration of the set operation
	 * @param user User name 
	 * @param password User Password
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public void execSetOperation(SetOperation setOp, String user, String password) throws OperationExecutionException {
		if(!controlService())
			return;
		execService.doSetOperation(setOp, user, password);
	}
	
	/**
	 * Execs a Get operation using a default user credentials
	 * @param getOp Configuration of the get operation
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public CmsNode execGetOperation(GetOperation getOp) throws OperationExecutionException {
		if(!controlService())
			return null;
		CmsNode node = execService.doGetOperation(getOp);
		return node;
	}
	
	/**
	 * Execs a get operation using a specific user name and password
	 * @param getOp Configuration of the get operation
	 * @param user User name 
	 * @param password User Password
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public CmsNode execGetOperation(GetOperation getOp, String user, String password) throws OperationExecutionException {
		if(!controlService())
			return null;
		CmsNode node = execService.doGetOperation(getOp, user, password);
		return node;
	}
	
	/**
	 * Execs a Search operation using a default user credentials
	 * @param searchOp Configuration of the search operation
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public List execSearchOperation(SearchOperation searchOp) throws OperationExecutionException {
		if(!controlService())
			return null;
		List nodes = execService.doSearchOperation(searchOp);
		return nodes;
	}
	
	/**
	 * Execs a search operation using a specific user name and password
	 * @param searchOp Configuration of the search operation
	 * @param user User name 
	 * @param password User Password
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public List execSearchOperation(SearchOperation searchOp, String user, String password) throws OperationExecutionException {
		if(!controlService())
			return null;
		List nodes = execService.doSearchOperation(searchOp, user, password);
		return nodes;
	}
	
	
	/**
	 * Execs a delete operation using a default user credentials
	 * @param delOp Configuration of the delete operation
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public void execDeleteOperation(DeleteOperation delOp) throws OperationExecutionException {
		if(!controlService())
			return;
		execService.doDeleteOperation(delOp);
	}
	
	/**
	 * Execs a delete operation using a specific user name and password
	 * @param delOp Configuration of the delete operation
	 * @param user User name 
	 * @param password User Password
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public void execDeleteOperation(DeleteOperation delOp, String user, String password) throws OperationExecutionException {
		if(!controlService())
			return;
		execService.doDeleteOperation(delOp, user, password);
	}
	
	/**
	 * Execs a restore operation using a default user credentials
	 * @param restOp Configuration of the restore operation
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public void execRestoreOperation(RestoreOperation restOp) throws OperationExecutionException {
		if(!controlService())
			return;
		execService.doRestoreOperation(restOp);
	}
	
	/**
	 * Execs a restore operation using a specific user name and password
	 * @param restOp Configuration of the restore operation
	 * @param user User name 
	 * @param password User Password
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public void execRestoreOperation(RestoreOperation restOp, String user, String password) throws OperationExecutionException {
		if(!controlService())
			return;
		execService.doRestoreOperation(restOp, user, password);
	}
	
	/**
	 * Execs an export operation using a default user credentials
	 * @param exportOp Configuration of the export operation
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public void execExportOperation(ExportOperation exportOp) throws OperationExecutionException {
		if(!controlService())
			return;
		execService.doExportOperation(exportOp);
	}
	
	/**
	 * Execs an export operation using a specific user name and password
	 * @param exportOp Configuration of the export operation
	 * @param user User name 
	 * @param password User Password
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public void execExportOperation(ExportOperation exportOp, String user, String password) throws OperationExecutionException {
		if(!controlService())
			return;
		execService.doExportOperation(exportOp, user, password);
	}
	
	/**
	 * Execs an import operation using a default user credentials
	 * @param importOp Configuration of the import operation
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public void execImportOperation(ImportOperation importOp) throws OperationExecutionException {
		if(!controlService())
			return;
		execService.doImportOperation(importOp);
	}
	
	/**
	 * Execs an import operation using a specific user name and password
	 * @param importOp Configuration of the import operation
	 * @param user User name 
	 * @param password User Password
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public void execImportOperation(ImportOperation importOp, String user, String password) throws OperationExecutionException {
		if(!controlService())
			return;
		execService.doImportOperation(importOp, user, password);
	}
	
	/**
	 * Execs a version operation using a default user credentials
	 * @param versionOp Configuration of the version operation
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public void execVersionOperation(VersionOperation versionOp) throws OperationExecutionException {
		if(!controlService())
			return;
		execService.doVersionOperation(versionOp);
	}
	
	/**
	 * Execs a version operation using a specific user name and password
	 * @param versionOp Configuration of the version operation
	 * @param user User name 
	 * @param password User Password
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public void execVersionOperation(VersionOperation versionOp, String user, String password) throws OperationExecutionException {
		if(!controlService())
			return;
		execService.doVersionOperation(versionOp, user, password);
	}
	
	/**
	 * Checks if the instance of the IExecutionService class has been created
	 * @return true if the instance has been created, flase otherwise
	 */
	private boolean controlService() {
		if(execService==null) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
        						"CmsManager::controlService: the execution service is null");
			return false;
		}else return true;
	}
	
	
}
