/**

    Copyright 2004, 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.cms;

import it.eng.spago.cms.constants.Constants;
import it.eng.spago.tracing.TracerSingleton;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

/**
 * Models the property of a cms node
 */
public class CmsProperty {

	/**
	 * Binary type of the property (java InputStream type)
	 */
	public final static String TYPE_BINARY = "BINARY";
	/**
	 * Boolean type of the property (java Boolean type)
	 */
	public final static String TYPE_BOOLEAN = "BOOLEAN";
	/**
	 * Date type of the property (java Calendar type)
	 */
	public final static String TYPE_DATE = "DATE";
	/**
	 * Double type of the property (java Double type)
	 */
	public final static String TYPE_DOUBLE = "DOUBLE";
	/**
	 * Long type of the property (java Long type)
	 */
	public final static String TYPE_LONG = "LONG";
	/**
	 * String type of the property (java String type)
	 */
	public final static String TYPE_STRING = "STRING";

	String name = "";
	String type = "";
	List values = new ArrayList();
	
	/**
	 * Constructor of the class useful to build a properties of type String
	 * setting a name and an array of string values
	 * @param namein The name of the property
	 * @param valuesin The array of String values
	 */
	public CmsProperty(String namein, String[] valuesin) {
		name = namein;
		type = TYPE_STRING;
		setStringValues(valuesin);
	}
	
	/**
	 * Constructor of the class useful to build a properties of type Long
	 * setting a name and an array of Long values
	 * @param namein The name of the property
	 * @param valuesin The array of Long values
	 */
	public CmsProperty(String namein, Long[] valuesin) {
		name = namein;
		type = TYPE_LONG;
		setLongValues(valuesin);
	}
	
	/**
	 * Constructor of the class useful to build a properties of type Double
	 * setting a name and an array of Double values
	 * @param namein The name of the property
	 * @param valuesin The array of Double values
	 */
	public CmsProperty(String namein, Double[] valuesin) {
		name = namein;
		type = TYPE_DOUBLE;
		setDoubleValues(valuesin);
	}
	
	/**
	 * Constructor of the class useful to build a properties of type Date
	 * setting a name and an array of Calendar values
	 * @param namein The name of the property
	 * @param valuesin The array of Calendar values
	 */
	public CmsProperty(String namein, Calendar[] valuesin) {
		name = namein;
		type = TYPE_DATE;
		setDateValues(valuesin);
	}
	
	/**
	 * Constructor of the class useful to build a properties of type Boolean
	 * setting a name and an array of Boolean values
	 * @param namein The name of the property
	 * @param valuesin The array of Boolean values
	 */
	public CmsProperty(String namein, Boolean[] valuesin) {
		name = namein;
		type = TYPE_BOOLEAN;
		setBooleanValues(valuesin);
	}
	
	/**
	 * Constructor of the class useful to build a properties of type Binary
	 * setting a name and an array of InputStream values
	 * @param namein The name of the property
	 * @param valuesin The array of InputStream values
	 */
	public CmsProperty(String namein, InputStream[] valuesin) {
		name = namein;
		type = TYPE_BINARY;
		setBinaryValues(valuesin);
	}
	
	
	/**
	 * Gets the name of the property
	 * @return the name of the property
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * Sets the name of the property
	 * @param name The name of the property
	 */
	public void setName(String name) {
		this.name = name;
	}
	
	/**
	 * Gets the type of the property
	 * @return the type of the property (BINARY/BOOLEAN/DOUBLE/DATE/LONG/STRING)
	 */
	public String getType() {
		return type;
	}
	
	/**
	 * Sets the type of the property
	 * @param type The type of the property
	 */
	public void setType(String type) {
		this.type = type;
	}

	/**
	 * Gets the String values of the property
	 * @return Array of the String values of the property
	 */
    public String[] getStringValues() {	
    	try {
    		int size = values.size();
    		String [] array = new String[size];
    		for(int i=0; i<size; i++) {
    			array[i] = (String)values.get(i);
    		}
    		return array;
    	} catch (Exception e) {
    		 TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.WARNING,
                                 "Property::getStringValues: property value cast exception");
    	     return new String[0];
    	}
    }
    
	/**
	 * Sets the String values of the property
	 * @param invalues Array of the String values of the property
	 */
    public void setStringValues(String[] invalues) {	
    	for(int i=0; i<invalues.length; i++)
    		values.add(invalues[i]);
    }
    
    
	/**
	 * Gets the Binary values of the property
	 * @return Array of the InputStream values of the property
	 */
    public InputStream[] getBinaryValues() {
    	try {
    		int size = values.size();
    		InputStream [] array = new InputStream[size];
    		for(int i=0; i<size; i++) {
    			array[i] = (InputStream)values.get(i);
    		}
    		return array;
    	} catch (Exception e) {
    		 TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.WARNING,
                                 "Property::getBinaryValues: property value cast exception");
    		 return new InputStream[0];
    	}
    }
    
    /**
	 * Sets the Binary values of the property
	 * @param invalues Array of the InputStream values of the property
	 */
    public void setBinaryValues(InputStream[] invalues) {	
    	for(int i=0; i<invalues.length; i++)
    		values.add(invalues[i]);
    }
    
   
    /**
	 * Gets the Long values of the property
	 * @return Array of the Long values of the property
	 */
    public Long[] getLongValues() {
    	try {
    		int size = values.size();
    		Long [] array = new Long[size];
    		for(int i=0; i<size; i++) {
    			array[i] = (Long)values.get(i);
    		}
    		return array;
    	} catch (Exception e) {
    		 TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.WARNING,
                                 "Property::getLongValues: property value cast exception");
             return new Long[0];
    	}
    }
    
    /**
	 * Sets the Long values of the property
	 * @param invalues Array of the Long values of the property
	 */
    public void setLongValues(Long[] invalues) {	
    	for(int i=0; i<invalues.length; i++)
    		values.add(invalues[i]);
    }
    
    
    /**
	 * Gets the Double values of the property
	 * @return Array of the Double values of the property
	 */
    public Double[] getDoubleValues() {
    	try {
    		int size = values.size();
    		Double [] array = new Double[size];
    		for(int i=0; i<size; i++) {
    			array[i] = (Double)values.get(i);
    		}
    		return array;
    	} catch (Exception e) {
    		 TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.WARNING,
                                 "Property::getDoubleValues: property value cast exception");
             return new Double[0];
    	}
    }
    
    /**
	 * Sets the Double values of the property
	 * @param invalues Array of the Double values of the property
	 */
    public void setDoubleValues(Double[] invalues) {	
    	for(int i=0; i<invalues.length; i++)
    		values.add(invalues[i]);
    }

    
    /**
	 * Gets the Boolean values of the property
	 * @return Array of the Boolean values of the property
	 */
    public Boolean[] getBooleanValues() {
    	try {
    		int size = values.size();
    		Boolean [] array = new Boolean[size];
    		for(int i=0; i<size; i++) {
    			array[i] = (Boolean)values.get(i);
    		}
    		return array;
    	} catch (Exception e) {
    		 TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.WARNING,
                                 "Property::getBooleanValues: property value cast exception");
             return new Boolean[0];
    	}
    }
    
    
    /**
	 * Sets the Boolean values of the property
	 * @param invalues Array of the Boolean values of the property
	 */
    public void setBooleanValues(Boolean[] invalues) {	
    	for(int i=0; i<invalues.length; i++)
    		values.add(invalues[i]);
    }
    
   
    /**
	 * Gets the Date values of the property
	 * @return Array of the Calendar values of the property
	 */
    public Calendar[] getDateValues() {
    	try {
    		int size = values.size();
    		Calendar [] array = new Calendar[size];
    		for(int i=0; i<size; i++) {
    			array[i] = (Calendar)values.get(i);
    		}
    		return array;
    	} catch (Exception e) {
    		TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.WARNING,
                                 "Property::getDateValues: property value cast exception");
            return new Calendar[0];
    	}
    }
	
    /**
	 * Sets the Date values of the property
	 * @param invalues Array of the Calendar values of the property
	 */
    public void setDateValues(Calendar[] invalues) {	
    	for(int i=0; i<invalues.length; i++)
    		values.add(invalues[i]);
    }
	
	
}
