/**

    Copyright 2004, 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.cms;

import java.util.List;

import it.eng.spago.cms.exceptions.OperationExecutionException;
import it.eng.spago.cms.operations.DeleteOperation;
import it.eng.spago.cms.operations.ExportOperation;
import it.eng.spago.cms.operations.GetOperation;
import it.eng.spago.cms.operations.ImportOperation;
import it.eng.spago.cms.operations.RestoreOperation;
import it.eng.spago.cms.operations.SearchOperation;
import it.eng.spago.cms.operations.SetOperation;
import it.eng.spago.cms.operations.VersionOperation;

/**
 * Defines the methods to exec operation on the repository. SpagoCms configuration must contain 
 * the name of a class that implements this interface
 */
public interface IExecutionService {

	/**
	 * Execs a Set operation using a default user credentials
	 * @param setOp Configuration of the set operation
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public void doSetOperation(SetOperation setOp) throws OperationExecutionException;
	/**
	 * Execs a Set operation using a specific user name and password
	 * @param setOp Configuration of the set operation
	 * @param user User name 
	 * @param password User Password
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public void doSetOperation(SetOperation setOp, String userId, String password) throws OperationExecutionException;
	
	/**
	 * Execs a Get operation using a default user credentials
	 * @param getOp Configuration of the get operation
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public CmsNode doGetOperation(GetOperation getOp) throws OperationExecutionException;
	/**
	 * Execs a get operation using a specific user name and password
	 * @param getOp Configuration of the get operation
	 * @param user User name 
	 * @param password User Password
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public CmsNode doGetOperation(GetOperation getOp, String userId, String password) throws OperationExecutionException;
	
	/**
	 * Execs a restore operation using a default user credentials
	 * @param restOp Configuration of the restore operation
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public void doRestoreOperation(RestoreOperation restOp) throws OperationExecutionException;
	/**
	 * Execs a restore operation using a specific user name and password
	 * @param restOp Configuration of the restore operation
	 * @param user User name 
	 * @param password User Password
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public void doRestoreOperation(RestoreOperation restOp, String userId, String password) throws OperationExecutionException;
	
	/**
	 * Execs a delete operation using a default user credentials
	 * @param delOp Configuration of the delete operation
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public void doDeleteOperation(DeleteOperation delOp) throws OperationExecutionException;
	/**
	 * Execs a delete operation using a specific user name and password
	 * @param delOp Configuration of the delete operation
	 * @param user User name 
	 * @param password User Password
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public void doDeleteOperation(DeleteOperation delOp, String userId, String password) throws OperationExecutionException;
	
	/**
	 * Execs a Search operation using a default user credentials
	 * @param searchOp Configuration of the search operation
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public List doSearchOperation(SearchOperation searchOp) throws OperationExecutionException;
	/**
	 * Execs a search operation using a specific user name and password
	 * @param searchOp Configuration of the search operation
	 * @param user User name 
	 * @param password User Password
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public List doSearchOperation(SearchOperation searchOp, String userId, String password) throws OperationExecutionException;
	
	/**
	 * Execs an export operation using a default user credentials
	 * @param exportOp Configuration of the export operation
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public void doExportOperation(ExportOperation exportOp) throws OperationExecutionException;
	/**
	 * Execs an export operation using a specific user name and password
	 * @param exportOp Configuration of the export operation
	 * @param user User name 
	 * @param password User Password
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public void doExportOperation(ExportOperation exportOp, String userId, String password) throws OperationExecutionException;
	
	/**
	 * Execs an import operation using a default user credentials
	 * @param importOp Configuration of the import operation
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public void doImportOperation(ImportOperation importOp) throws OperationExecutionException;
	/**
	 * Execs an import operation using a specific user name and password
	 * @param importOp Configuration of the import operation
	 * @param user User name 
	 * @param password User Password
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public void doImportOperation(ImportOperation importOp, String userId, String password) throws OperationExecutionException;
	
	/**
	 * Execs a version operation using a default user credentials.
	 * It means that the node relevant to the version operation is versioned only
	 * if it is versionable; if the node isn't versionable, an OperationExecutionException is thrown.
	 * @param versionOp Configuration of the version operation
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public void doVersionOperation(VersionOperation versionOp) throws OperationExecutionException;
	/**
	 * Execs an version operation using a specific user name and password
	 * It means that the node relevant to the version operation is versioned only
	 * if it is versionable; if the node isn't versionable, an OperationExecutionException is thrown.
	 * @param versionOp Configuration of the version operation
	 * @param user User name 
	 * @param password User Password
	 * @throws OperationExecutionException Throwing if an error occurs during the execution
	 */
	public void doVersionOperation(VersionOperation versionOp, String userId, String password) throws OperationExecutionException;
}
