/**

    Copyright 2004, 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.cms.operations;

import it.eng.spago.base.SourceBean;
import it.eng.spago.cms.exceptions.BuildOperationException;
import it.eng.spago.cms.constants.Constants;
import it.eng.spago.tracing.TracerSingleton;

/**
 * Models a repository delete operation. The operation has two attributes:
 * <ul>
 * 		<li>path: the path of the cms node to delete (mandatory)</li>
 * 		<li>version: the version of the cms node to delete (not mandatory)</li>
 * </ul>
 * To delete an entire node it's necessary to set only the path attribute. 
 * To delete only a version of a node it's necessary to set the path and the version 
 * name of the node.
 * The configuration of the operation is stored into a Spago SourceBean structured like below:
 * <pre>
 * &lt;OPERATION name=""&gt;
 *		&lt;DELETEOPERATION path="?" version="?" /&gt;
 * &lt;/OPERATION&gt;
 * </pre> 
 */
public class DeleteOperation {

	/**
	 * Internal structure of the operation
	 */
	private SourceBean operationDescriptor = null;
	
	
	/**
	 * Constructor of the class useful to build an empty delete operation.
	 * @throws BuildOperationException Thrown if an error occurs during the creation
	 */
	public DeleteOperation() throws BuildOperationException{	
		try {
			operationDescriptor = new SourceBean(Constants.NAME_DELETE_OPERATION);
			operationDescriptor.setAttribute(Constants.PATH, "");
			operationDescriptor.setAttribute(Constants.VERSION, "");
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "DeleteOperation::<init>(): Error during the creation of the delete operation" +
                                "SourceBean");
			throw new BuildOperationException("DeleteOperation::<init>(): Error during the creation of the " +
					                          "delete operation SourceBean", e);
		}
	}
	
	/**
	 * Constructor of the class useful to build a delete operation
	 * setting the path attribute.
	 * @param path The Path of the cms node to delete
	 * @throws BuildOperationException Thrown if an error occurs during the creation
	 */
	public DeleteOperation(String path) throws BuildOperationException{	
		try {
			operationDescriptor = new SourceBean(Constants.NAME_DELETE_OPERATION);
			operationDescriptor.setAttribute(Constants.PATH, path);
			operationDescriptor.setAttribute(Constants.VERSION, "");
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "DeleteOperation::<init>(String): Error during the creation of the delete operation" +
                                "SourceBean");
			throw new BuildOperationException("DeleteOperation::<init>(String): Error during the creation of the " +
					                          "delete operation SourceBean", e);
		}
	}
	
	/**
	 * Constructor of the class useful to build a delete operation
	 * setting the path and version attributes.
	 * @param path The Path of the cms node 
	 * @param version The name of the node version to delete  
	 * @throws BuildOperationException Thrown if an error occurs during the creation
	 */
	public DeleteOperation(String path, String version) throws BuildOperationException{	
		try {
			operationDescriptor = new SourceBean(Constants.NAME_DELETE_OPERATION);
			operationDescriptor.setAttribute(Constants.PATH, path);
			operationDescriptor.setAttribute(Constants.VERSION, version);
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "DeleteOperation::<init>(String, String): Error during the creation of the delete operation" +
                                "SourceBean");
			throw new BuildOperationException("DeleteOperation::<init>(String, String): Error during the creation of the " +
					                          "delete operation SourceBean", e);
		}
	}
	
	/**
	 * Gets the Internal SourceBean which defines the internal structure
	 * of the operation
	 * @return The Internal SourceBean of the operation which defines
	 * the internal structure of the operation
	 */
	public SourceBean getOperationDescriptor() {
		return operationDescriptor;
	}

	/**
	 * Set the path of the cms node
	 * @param path Path of the cms node
	 * @throws BuildOperationException Thrown if an error occurs during the setting
	 */
	public void setPath(String path) throws BuildOperationException {
		try {
			operationDescriptor.updAttribute(Constants.PATH, path);
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "DeleteOperation::setPath: Error during the path setting");
            throw new BuildOperationException("DeleteOperation::setPath: Error during the path setting", e);
		}
	}
	
	/**
	 * Set the version of the cms node
	 * @param version Version of the cms node
	 * @throws BuildOperationException Thrown if an error occurs during the setting
	 */
	public void setVersion(String version) throws BuildOperationException {
		try {
			operationDescriptor.updAttribute(Constants.VERSION, version);
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "DeleteOperation::setVersion: Error during the version setting");
            throw new BuildOperationException("DeleteOperation::setVersion: Error during the version setting", e);
		}
	}
	
	
}
