package it.eng.spago.cms.operations;

import it.eng.spago.base.SourceBean;
import it.eng.spago.cms.constants.Constants;
import it.eng.spago.cms.exceptions.BuildOperationException;
import it.eng.spago.tracing.TracerSingleton;

import java.io.ByteArrayOutputStream;
import java.io.OutputStream;

/**
 * Models a repository export operation. The operation has three attributes:
 * <ul>
 * 		<li>path: the path of the cms node to be exported (mandatory)</li>
 * 		<li>stream: the output stream (java OutputStream object) where nodes are serialized (mandatory)</li>
 * 		<li>noRecurse: flag: if it is true then only the node at path and its properties,
 * 			but not its child nodes, are serialized; if it is false then the entire subtree rooted at
 * 			path is serialized (mandatory)</li>
 * </ul>
 * The configuration of the operation is stored into a Spago SourceBean structured like below:
 * <pre>
 * &lt;OPERATION name="EXPORT"&gt;
 * 	    &lt;EXPORTOPERATION path="?" stream="?" noRecurse="?" /&gt;  
 * &lt;/OPERATION&gt;
 * <pre>
 */
public class ExportOperation {

	/**
	 * Internal structure of the operation
	 */
	private SourceBean operationDescriptor = null;
	
	/**
	 * Constructor of the class useful to build an empty export operation.
	 * @throws BuildOperationException Thrown if an error occurs during the creation
	 */
	public ExportOperation() throws BuildOperationException {
		try {
			operationDescriptor = new SourceBean(Constants.NAME_EXPORT_OPERATION);
			operationDescriptor.setAttribute(Constants.PATH, "");
			operationDescriptor.setAttribute(Constants.STREAMATTR, new ByteArrayOutputStream());
			operationDescriptor.setAttribute(Constants.NORECURSE, "false");
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "ExportOperation::<init>(): Error during the creation of the export operation" +
                                "SourceBean");
			throw new BuildOperationException("ExportOperation::<init>(): Error during the creation of the " +
					                          "export operation SourceBean", e);
		}
	}
	
	/**
	 * Constructor of the class useful to build an export operation
	 * setting the path, stream and noRestore attributes.
	 * @param path The Path of the cms node to be exported
	 * @param stream The OutputStream for the nodes serialization
	 * @param noRecurse Flag: if it is true then only the node at path and its properties,
	 * 		  but not its child nodes, are serialized; if it is false then the entire subtree rooted at
	 * 		  path is serialized
	 * @throws BuildOperationException Thrown if an error occurs during the creation
	 */
	public ExportOperation(String path, OutputStream stream, boolean noRecurse) throws BuildOperationException {
		try {
			operationDescriptor = new SourceBean(Constants.NAME_EXPORT_OPERATION);
			operationDescriptor.setAttribute(Constants.PATH, path);
			operationDescriptor.setAttribute(Constants.STREAMATTR, stream);
			if (noRecurse) operationDescriptor.setAttribute(Constants.NORECURSE, "true");
			else operationDescriptor.setAttribute(Constants.NORECURSE, "false");
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "ExportOperation::<init>(String path, OutputStream stream, boolean noRecurse): " +
                                "Error during the creation of the export operation" +
                                "SourceBean");
			throw new BuildOperationException("ExportOperation::<init>(String path, OutputStream stream, " +
								"boolean noRecurse): Error during the creation of the " +
					            "export operation SourceBean", e);
		}
	}
	
	/**
	 * Set the path of the cms node to the exported
	 * @param path Path of the cms node to the exported
	 * @throws BuildOperationException Thrown if an error occurs during the setting
	 */
	public void setPath(String path) throws BuildOperationException {
		try {
			operationDescriptor.updAttribute(Constants.PATH, path);
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "ExportOperation::setPath: Error during the path setting");
            throw new BuildOperationException("ExportOperation::setPath: Error during the path setting", e);
		}
	}
	
	/**
	 * Set the noRecurse flag
	 * @param noRecurse The noRecurse flag to set
	 * @throws BuildOperationException Thrown if an error occurs during the setting
	 */
	public void setNoRecurse(boolean noRecurse) throws BuildOperationException {
		try {
			if (noRecurse) operationDescriptor.updAttribute(Constants.NORECURSE, "true");
			else operationDescriptor.updAttribute(Constants.NORECURSE, "false");
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "ExportOperation::setContent: Error during the noRecurse flag setting");
            throw new BuildOperationException("ExportOperation::setContent: Error during the noRecurse flag " +
            					"setting", e);
		}
	}
	
	/**
	 * Set the output stream for the nodes serialization
	 * @param stream OutputStream for the cms nodes serialization
	 * @throws BuildOperationException Thrown if an error occurs during the setting
	 */
	public void setOutputStream(OutputStream stream) throws BuildOperationException {
		try {
			operationDescriptor.updAttribute(Constants.STREAMATTR, stream);
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "ExportOperation::setContent: Error during the output stream setting");
            throw new BuildOperationException("ExportOperation::setContent: Error during the output stream " +
            					"setting", e);
		}
	}
	
	/**
	 * Gets the Internal SourceBean which defines the internal structure
	 * of the operation
	 * @return The Internal SourceBean of the operation which defines
	 * the internal structure of the operation
	 */
	public SourceBean getOperationDescriptor() {
		return operationDescriptor;
	}
	
}
