/**

    Copyright 2004, 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.cms.operations;

import it.eng.spago.base.SourceBean;
import it.eng.spago.cms.constants.Constants;
import it.eng.spago.cms.exceptions.BuildOperationException;
import it.eng.spago.tracing.TracerSingleton;

/**
 * Models an inquiry operation on the repostiroty . The operation has six attributes:
 * <ul>
 * 		<li>path: the path of the cms node where the inquiry starts (mandatory)</li>
 * 		<li>version: the version of the cms node to examine (not mandatory)</li>
 * 		<li>getChilds: to get information about the node childs (default false, not mandatory)</li>
 * 		<li>getContent: to get the content of the node (default false, not mandatory)</li>
 * 		<li>getProperties: to get information about node properties (default false, not mandatory)</li>
 * 		<li>getVersion: to get information about node versions (default false, not mandatory)</li>
 * </ul>
 * To gather information about the current version of an node it's
 * necessary to set only the path attribute. 
 * To gather information about a specific version of a node it's 
 * necessary to set the path and the version name of the node.
 * The configuration of the operation is stored into a Spago SourceBean structured like below:
 * <pre>
 * &lt;OPERATION name=""&gt;
 * 	    &lt;GETOPERATION path="?" version="?" getVersions="(true/false)" getChilds="(true/false)" 
 * 	    			  getContent="(true/false)" getProperties="(true/false)" /&gt;
 * &lt;/OPERATION&gt;
 * </pre>
 */
public class GetOperation {

	/**
	 * Internal structure of the operation
	 */
	private SourceBean operationDescriptor = null;
	
	/**
	 * Constructor of the class useful to build an empty get operation. 
	 * Attributes which have a default value are filled with it 
	 * @throws BuildOperationException Thrown if an error occurs during the creation
	 */
	public GetOperation() throws BuildOperationException {	
		try {
			operationDescriptor = new SourceBean(Constants.NAME_GET_OPERATION);
			operationDescriptor.setAttribute(Constants.PATH, "");
			operationDescriptor.setAttribute(Constants.VERSION, "");
			operationDescriptor.setAttribute(Constants.GETCHILDS, "false");
			operationDescriptor.setAttribute(Constants.GETCONTENT, "false");
			operationDescriptor.setAttribute(Constants.GETPROPERTIES, "false");
			operationDescriptor.setAttribute(Constants.GETVERSIONS, "false");
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "GetOperation::<init>(): Error during the creation of the get operation" +
                                "SourceBean");
			throw new BuildOperationException("GetOperation::<init>(): Error during the creation of the " +
					                          "get operation SourceBean", e);
		}
	}
	
	
	/**
	 * Constructor of the class useful to build a get operation
	 * setting the path, getChilds, getProperties, getVersions, 
	 * getContent attributes.
	 * @param path The Path of the cms node to examine
	 * @param getChild Flag to activate or deactivate the child information gathering
	 * @param getProps Flag to activate or deactivate the properties information gathering
	 * @param getCont Flag to activate or deactivate the content gathering
	 * @param getVers Flag to activate or deactivate the version information gathering
	 * @throws BuildOperationException Thrown if an error occurs during the creation
	 */
	public GetOperation(String path, boolean getChild, boolean getProps, 
			            boolean getCont, boolean getVers) throws BuildOperationException {	
		try {
			operationDescriptor = new SourceBean(Constants.NAME_GET_OPERATION);
			operationDescriptor.setAttribute(Constants.PATH, path);
			operationDescriptor.setAttribute(Constants.VERSION, "");
			if(getChild)
				operationDescriptor.setAttribute(Constants.GETCHILDS, "true");
			else 
				operationDescriptor.setAttribute(Constants.GETCHILDS, "false");
			if(getCont)
				operationDescriptor.setAttribute(Constants.GETCONTENT, "true");
			else 
				operationDescriptor.setAttribute(Constants.GETCONTENT, "false");
			if(getProps)
				operationDescriptor.setAttribute(Constants.GETPROPERTIES, "true");
			else 
				operationDescriptor.setAttribute(Constants.GETPROPERTIES, "false");
            if(getVers)
            	operationDescriptor.setAttribute(Constants.GETVERSIONS, "true");
            else 
            	operationDescriptor.setAttribute(Constants.GETVERSIONS, "false");
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "GetOperation::<init>(String, boolean, boolean, boolean, boolean): " +
                                "Error during the creation of the get operation" +
                                "SourceBean");
			throw new BuildOperationException("GetOperation::<init>(String, boolean, boolean, boolean, boolean): " +
											  "Error during the creation of the " +
					                          "get operation SourceBean", e);
		}
	}
	
	
	
	/**
	 * Constructor of the class useful to build a get operation
	 * setting the path, version, getChilds, getProperties, getVersions, 
	 * getContent attributes.
	 * @param path The Path of the cms node
	 * @param version The version of the node to examine
	 * @param getChild Flag to activate or deactivate the child information gathering
	 * @param getProps Flag to activate or deactivate the properties information gathering
	 * @param getCont Flag to activate or deactivate the content gathering
	 * @param getVers Flag to activate or deactivate the version information gathering
	 * @throws BuildOperationException Thrown if an error occurs during the creation
	 */
	public GetOperation(String path, String version, boolean getChild, boolean getProps, 
            			boolean getCont, boolean getVers) throws BuildOperationException {	
		try {
			operationDescriptor = new SourceBean(Constants.NAME_GET_OPERATION);
			operationDescriptor.setAttribute(Constants.PATH, path);
			operationDescriptor.setAttribute(Constants.VERSION,	version);
			if(getChild)
				operationDescriptor.setAttribute(Constants.GETCHILDS, "true");
			else 
				operationDescriptor.setAttribute(Constants.GETCHILDS, "false");
			if(getCont)
				operationDescriptor.setAttribute(Constants.GETCONTENT, "true");
			else 
				operationDescriptor.setAttribute(Constants.GETCONTENT, "false");
			if(getProps)
				operationDescriptor.setAttribute(Constants.GETPROPERTIES, "true");
			else 
				operationDescriptor.setAttribute(Constants.GETPROPERTIES, "false");
			if(getVers)
				operationDescriptor.setAttribute(Constants.GETVERSIONS, "true");
			else 
				operationDescriptor.setAttribute(Constants.GETVERSIONS, "false");
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                    "GetOperation::<init>(String, String, boolean, boolean, boolean, boolean): " +
                    "Error during the creation of the get operation" +
                    "SourceBean");
			throw new BuildOperationException("GetOperation::<init>(String, String, boolean, boolean, boolean, boolean): " +
								  "Error during the creation of the " +
		                          "get operation SourceBean", e);
		}
	}
	
	
	/**
	 * Gets the Internal SourceBean which defines the internal structure
	 * of the operation
	 * @return The Internal SourceBean of the operation which defines
	 * the internal structure of the operation
	 */
	public SourceBean getOperationDescriptor() {
		return operationDescriptor;
	}

	
	/**
	 * Set the path of the cms node
	 * @param path Path of the cms node
	 * @throws BuildOperationException Thrown if an error occurs during the setting
	 */
	public void setPath(String path) throws BuildOperationException {
		try {
			operationDescriptor.updAttribute(Constants.PATH, path);
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "GetOperation::setPath: Error during the path setting");
            throw new BuildOperationException("GetOperation::setPath: Error during the path setting", e);
		}
	}
	
	/**
	 * Set the versione of the cms node
	 * @param version Version of the cms node
	 * @throws BuildOperationException Thrown if an error occurs during the setting
	 */
	public void setVersion(String ver) throws BuildOperationException {
		try {
			operationDescriptor.updAttribute(Constants.VERSION, ver);
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "GetOperation::setVersion: Error during the vversion setting");
            throw new BuildOperationException("GetOperation::setVersion: Error during the version setting", e);
		}
	}
	
	
	/**
	 * Set the getVersions attribute of the get operation
	 * @param getVersion Flag to activate or deactivate the version information gathering
	 * @throws BuildOperationException Thrown if an error occurs during the setting
	 */
	public void setRetriveVersionsInformation(String getVersion) throws BuildOperationException {
		try {
			operationDescriptor.updAttribute(Constants.GETVERSIONS, getVersion);
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "GetOperation::setRetriveVersionsInformation: Error during the getVersion setting");
            throw new BuildOperationException("GetOperation::setRetriveVersionsInformation: Error during the getVersion setting", e);
		}
	}
	
	/**
	 * Set the getContent attribute of the get operation
	 * @param getContent Flag to activate or deactivate the content gathering
	 * @throws BuildOperationException Thrown if an error occurs during the setting
	 */
	public void setRetriveContentInformation(String getContent) throws BuildOperationException {
		try {
			operationDescriptor.updAttribute(Constants.GETCONTENT, getContent);
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "GetOperation::setRetriveContentInformation: Error during the getContent setting");
            throw new BuildOperationException("GetOperation::setRetriveContentInformation: Error during the getContent setting", e);
		}
	}

	
	/**
	 * Set the getProperties attribute of the get operation
	 * @param getProperties Flag to activate or deactivate the information gathering about node properties
	 * @throws BuildOperationException Thrown if an error occurs during the setting
	 */
	public void setRetrivePropertiesInformation(String getProperties) throws BuildOperationException {
		try {
			operationDescriptor.updAttribute(Constants.GETPROPERTIES, getProperties);
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "GetOperation::setRetrivePropertiesInformation: Error during the getProperties setting");
            throw new BuildOperationException("GetOperation::setRetrivePropertiesInformation: Error during the getProperties setting", e);
		}
	}
	
	/**
	 * Set the getChilds attribute of the get operation
	 * @param getChilds Flag to activate or deactivate the information gathering about node childs
	 * @throws BuildOperationException Thrown if an error occurs during the setting
	 */
	public void setRetriveChildsInformation(String getChilds) throws BuildOperationException {
		try {
			operationDescriptor.updAttribute(Constants.GETCHILDS, getChilds);
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "GetOperation::setRetriveChildsInformation: Error during the getChilds setting");
            throw new BuildOperationException("GetOperation::setRetriveChildsInformation: Error during the getChilds setting", e);
		}
	}
}
