package it.eng.spago.cms.operations;

import it.eng.spago.base.SourceBean;
import it.eng.spago.cms.constants.Constants;
import it.eng.spago.cms.exceptions.BuildOperationException;
import it.eng.spago.tracing.TracerSingleton;

import java.io.ByteArrayInputStream;
import java.io.InputStream;

import javax.jcr.ImportUUIDBehavior;

/**
 * Models a repository import operation. The operation has three attributes:
 * <ul>
 * 		<li>path: the path of the cms node where importing nodes will be placed (mandatory)</li>
 * 		<li>stream: the input stream (java InputStream object) where importing nodes are deserialized 
 * 			from (mandatory)</li>
 * 		<li>uuidBehavior: int flag that governs how the UUIDs of incoming nodes are handled:
 * 			see javax.jcr.ImportUUIDBehavior (mandatory)</li>
 * </ul>
 * The configuration of the operation is stored into a Spago SourceBean structured like below:
 * <pre>
 * &lt;OPERATION name="IMPORT"&gt;
 * 	    &lt;IMPORTOPERATION path="?" stream="?" uuidBehavior="?" /&gt;  
 * &lt;/OPERATION&gt;
 * <pre>
 */
public class ImportOperation {

	/**
	 * Internal structure of the operation
	 */
	private SourceBean operationDescriptor = null;
	
	/**
	 * Constructor of the class useful to build an empty import operation.
	 * @throws BuildOperationException Thrown if an error occurs during the creation
	 */
	public ImportOperation() throws BuildOperationException {
		try {
			operationDescriptor = new SourceBean(Constants.NAME_IMPORT_OPERATION);
			operationDescriptor.setAttribute(Constants.PATH, "");
			operationDescriptor.setAttribute(Constants.STREAMATTR, new ByteArrayInputStream(new byte[0]));
			operationDescriptor.setAttribute(Constants.UUIDBEHAVIOR, 
					new Integer(ImportUUIDBehavior.IMPORT_UUID_CREATE_NEW).toString());
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "ImportOperation::<init>(): Error during the creation of the import operation" +
                                "SourceBean");
			throw new BuildOperationException("ImportOperation::<init>(): Error during the creation of the " +
					                          "import operation SourceBean", e);
		}
	}
	
	/**
	 * Constructor of the class useful to build an import operation
	 * setting the path, stream and uuidBehavior attributes.
	 * @param path The path of the cms node where importing nodes will be placed
	 * @param stream The InputStream where importing nodes are deserialized from
	 * @param uuidBehavior Int flag that governs how the UUIDs of incoming nodes are handled:
	 * 		  see javax.jcr.ImportUUIDBehavior
	 * @throws BuildOperationException Thrown if an error occurs during the creation
	 */
	public ImportOperation(String path, InputStream stream, int uuidBehavior) throws BuildOperationException {
		try {
			operationDescriptor = new SourceBean(Constants.NAME_IMPORT_OPERATION);
			operationDescriptor.setAttribute(Constants.PATH, path);
			operationDescriptor.setAttribute(Constants.STREAMATTR, stream);
			operationDescriptor.setAttribute(Constants.UUIDBEHAVIOR, new Integer(uuidBehavior).toString());
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "ImportOperation::<init>(String path, InputStream stream, int uuidBehavior): " +
                                "Error during the creation of the import operation" +
                                "SourceBean");
			throw new BuildOperationException("ImportOperation::<init>(String path, InputStream stream, " +
								"int uuidBehavior): Error during the creation of the " +
					            "import operation SourceBean", e);
		}
	}
	
	/**
	 * Constructor of the class useful to build an import operation
	 * setting the path and stream. 
	 * For uuidBehavior the default value is considered, that is: 
	 * javax.jcr.ImportUUIDBehavior.IMPORT_UUID_CREATE_NEW.
	 * @param path The path of the cms node where importing nodes will be placed
	 * @param stream The InputStream where importing nodes are deserialized from
	 * @param uuidBehavior Int flag that governs how the UUIDs of incoming nodes are handled:
	 * 		  see javax.jcr.ImportUUIDBehavior
	 * @throws BuildOperationException Thrown if an error occurs during the creation
	 */
	public ImportOperation(String path, InputStream stream) throws BuildOperationException {
		try {
			operationDescriptor = new SourceBean(Constants.NAME_IMPORT_OPERATION);
			operationDescriptor.setAttribute(Constants.PATH, path);
			operationDescriptor.setAttribute(Constants.STREAMATTR, stream);
			operationDescriptor.setAttribute(Constants.UUIDBEHAVIOR, 
					new Integer(ImportUUIDBehavior.IMPORT_UUID_CREATE_NEW).toString());
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "ImportOperation::<init>(String path, InputStream stream, int uuidBehavior): " +
                                "Error during the creation of the import operation" +
                                "SourceBean");
			throw new BuildOperationException("ImportOperation::<init>(String path, InputStream stream, " +
								"int uuidBehavior): Error during the creation of the " +
					            "import operation SourceBean", e);
		}
	}
	
	/**
	 * Set the path of the cms node where importing nodes will be placed
	 * @param path The path of the cms node where importing nodes will be placed
	 * @throws BuildOperationException Thrown if an error occurs during the setting
	 */
	public void setPath(String path) throws BuildOperationException {
		try {
			operationDescriptor.updAttribute(Constants.PATH, path);
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "ImportOperation::setPath: Error during the path setting");
            throw new BuildOperationException("ImportOperation::setPath: Error during the path setting", e);
		}
	}
	
	/**
	 * Set the uuidBehavior int flag: see javax.jcr.ImportUUIDBehavior
	 * @param uuidBehavior The uuidBehavior int flag to set
	 * @throws BuildOperationException Thrown if an error occurs during the setting
	 */
	public void setUuidBehavior(int uuidBehavior) throws BuildOperationException {
		try {
			operationDescriptor.updAttribute(Constants.UUIDBEHAVIOR, new Integer(uuidBehavior).toString());
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "ImportOperation::setContent: Error during the uuidBehavior setting");
            throw new BuildOperationException("ImportOperation::setContent: Error during the uuidBehavior " +
            					"setting", e);
		}
	}
	
	/**
	 * Set the input stream for the nodes deserialization
	 * @param stream InputStream for the cms nodes deserialization
	 * @throws BuildOperationException Thrown if an error occurs during the setting
	 */
	public void setInputStream(InputStream stream) throws BuildOperationException {
		try {
			operationDescriptor.updAttribute(Constants.STREAMATTR, stream);
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "ImportOperation::setContent: Error during the input stream setting");
            throw new BuildOperationException("ImportOperation::setContent: Error during the input stream " +
            					"setting", e);
		}
	}
	
	/**
	 * Gets the Internal SourceBean which defines the internal structure
	 * of the operation
	 * @return The Internal SourceBean of the operation which defines
	 * the internal structure of the operation
	 */
	public SourceBean getOperationDescriptor() {
		return operationDescriptor;
	}
	
}
