/**

    Copyright 2004, 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.cms.operations;

import it.eng.spago.base.SourceBean;
import it.eng.spago.cms.exceptions.BuildOperationException;
import it.eng.spago.cms.constants.Constants;
import it.eng.spago.tracing.TracerSingleton;


/**
 * Models a repository search operation. The operation has two attributes:
 * <ul>
 * 		<li>query: the inquiry to exec on the repository. 
 *          The inquiry can be written using two different grammars: 
 *          xpath and jsr 170 specific grammar </li>
 * 		<li>language: the language used to write the query. Possible values are
 *          'xpath' (xPath languange) or 'sql' (jsr 170 specific grammar)
 *          (default value xpath, not mandatory)</li>
 * </ul>
 * The configuration of the operation is stored into a Spago SourceBean structured like below:
 * <pre>
 * &lt;OPERATION name=""&gt;
 * 	    &lt;SEARCHOPERATION query="?" language="(xpath/sql)" /&gt;
 * &lt;/OPERATION&gt;
 * </pre>
 */
public class SearchOperation {

	/**
	 * Internal structure of the operation
	 */
	private SourceBean operationDescriptor = null;
	
	
	/**
	 * Constructor of the class useful to build an empty search operation. 
	 * Attributes which have a default value are filled with it 
	 * @throws BuildOperationException Thrown if an error occurs during the creation
	 */
	public SearchOperation() throws BuildOperationException {	
		try {
			operationDescriptor = new SourceBean(Constants.NAME_SEARCH_OPERATION);
			operationDescriptor.setAttribute(Constants.QUERY_ATTRIBUTE, "");
			operationDescriptor.setAttribute(Constants.LANGUAGE_ATTRIBUTE, "xpath");
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "SearchOperation::<init>(): Error during the creation of the search operation" +
                                "SourceBean");
			throw new BuildOperationException("SearchOperation::<init>(): Error during the creation of the " +
					                          "search operation SourceBean", e);
		}
	}
	
	/**
	 * Constructor of the class useful to build a search operation
	 * setting the query and language type attributes.
	 * @param query The query to exec on the repository 
	 * @param language The label of the query language 
	 * (xpath = xPath grammar, sql = jsr170 grammar)  
	 * @throws BuildOperationException Thrown if an error occurs during the setting
	 */
	public SearchOperation(String query, String language) throws BuildOperationException {	
		try {
			operationDescriptor = new SourceBean(Constants.NAME_SEARCH_OPERATION);
			operationDescriptor.setAttribute(Constants.QUERY_ATTRIBUTE, query);
			operationDescriptor.setAttribute(Constants.LANGUAGE_ATTRIBUTE, language);
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "SearchOperation::<init>(): Error during the creation of the search operation" +
                                "SourceBean");
			throw new BuildOperationException("SearchOperation::<init>(String, String): " +
											  "Error during the creation of the " +
					                          "search operation SourceBean", e);
		}
	}
		
	/**
	 * Gets the Internal SourceBean which defines the internal structure
	 * of the operation
	 * @return The Internal SourceBean of the operation which defines
	 * the internal structure of the operation
	 */
	public SourceBean getOperationDescriptor() {
		return operationDescriptor;
	}

	/**
	 * Set the query to exec
	 * @param query String of the Query to exec written using xpath or jsr170 grammar 
	 * @throws BuildOperationException Thrown if an error occurs during the setting
	 */
	public void setQuery(String query) throws BuildOperationException {
		try {
			operationDescriptor.updAttribute(Constants.QUERY_ATTRIBUTE, query);
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "SearchOperation::setQuery: Error during the query setting");
            throw new BuildOperationException("SearchOperation::setQuery: Error during the path setting", e);
		}
	}
	
	/**
	 * Set the language type of the query
	 * @param language Grammar language used to define the query 
	 * (xpath = xPath grammar, sql = jsr170 grammar)  
	 * @throws BuildOperationException Thrown if an error occurs during the setting
	 */
	public void setLanguage(String language) throws BuildOperationException {
		try {
			operationDescriptor.updAttribute(Constants.LANGUAGE_ATTRIBUTE, language);
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "SearchOperation::setLanguage: Error during the language setting");
            throw new BuildOperationException("SearchOperation::setLanguage: Error during the language setting", e);
		}
	}
	
}
