/**

    Copyright 2004, 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.cms.operations;

import it.eng.spago.cms.CmsProperty;
import it.eng.spago.cms.exceptions.BuildOperationException;
import it.eng.spago.cms.constants.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.tracing.TracerSingleton;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;

/**
 * Models a set operation on the repostitory. The operation has seven attributes:
 * <ul>
 * 		<li>path: the path of the cms node to set. If the node doesn't exist 
 *          it will be created otherwise will be created a new version (mandatory)</li>
 * 		<li>type: the type of the node. The possible values are SetOperation.TYPE_CONTENT
 *          which is a leaf node with a content (file) or SetOperation.TYPE_CONTAINER which is
 *          not a leaf node, it doesn't contains content but it can have childs
 *          (folder). The defalut value of the attribute is SetOperation.TYPE_CONTENT</li>
 * 		<li>cancelOldProperties: Flag to activate or deactivate the deletion of the 
 *          old properties of the node in case a new version will be created
 *          (default value 'false')</li>
 * 		<li>versionable: Flag that indicates if the node must be versionable or not
 *          in case it does not exist (default value 'false')</li>
 * </ul>
 * If the type of the node to set is SetOperation.TYPE_CONTENT it's necessary to set also 
 * the content to store into the node. 
 * It's possible to set one or more properties for the node. Each property as a name, a type 
 * and a set of values congruent with the type. The possible types oF a properties are: 
 * STRING (java String object), DATE(java Calendar object), LONG(java Long object), 
 * DOUBLE(java Double object), BOOLEAN (java Boolean object), BINARY(java InputStream object)
 * The configuration of the operation is stored into a Spago SourceBean structured like below:
 * <pre>
 * &lt;OPERATION name=""&gt;
 * 		&lt;SETOPERATION path="" type="(container/content)" cancelOldProperties="(true/flase)"&gt;
 * 		 	&lt;CONTENT stream="" /&gt; (optional)
 * 		 	&lt;PROPERTIES&gt; (optional)
 * 		 		&lt;PROPERTY name="" type="STRING/DATE/LONG/DOUBLE/BOOLEAN/BINARY" &gt;
 * 		 			&lt;PROPERTYVALUE value=""/&gt;
 * 		 			&lt;PROPERTYVALUE value=""/&gt;
 * 		 			....
 * 		 		&lt;/PROPERTY&gt;
 * 		 		....
 * 		 	&lt;/PROPERTIES&gt;
 * 		&lt;/SETOPERATION&gt;
 * &lt;/OPERATION&gt;
 * </pre> 
 */
public class SetOperation {

	/**
	 * The property content type (leaf node) 
	 */
	public final static String TYPE_CONTENT = "content";
	/**
	 * the property container type
	 */
	public final static String TYPE_CONTAINER = "container";
	/**
	 * Internal structure of the operation
	 */
	private SourceBean operationDescriptor = null;
		
	
	/**
	 * Constructor of the class useful to build an empty set operation. 
	 * Attributes which have a default value are filled with it 
	 * @throws BuildOperationException Thrown if an error occurs during the creation
	 */
	public SetOperation() throws BuildOperationException {	
		try {
			operationDescriptor = new SourceBean(Constants.NAME_SET_OPERATION);
			operationDescriptor.setAttribute(Constants.PATH, "");
			operationDescriptor.setAttribute(Constants.TYPE, "");
			SourceBean contentSB = new SourceBean(Constants.CONTENTSB);
			contentSB.setAttribute(Constants.STREAMATTR, new ByteArrayInputStream(new byte[0]));
			operationDescriptor.setAttribute(Constants.OLDPROPERTIES, "false");
			SourceBean props = new SourceBean(Constants.PROPERTIES);
			operationDescriptor.setAttribute(props);
			operationDescriptor.setAttribute(Constants.VERSIONABLE, "false");
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "SetOperation::<init>(): Error during the creation of the set operation" +
                                "SourceBean");
			throw new BuildOperationException("SetOperation::<init>(): Error during the creation of the " +
					                          "set operation SourceBean", e);
		}
	}
	
	/**
	 * Constructor of the class useful to build a set operation
	 * setting the path, type and cancelOldProperties attributes.
	 * @param path The Path of the cms node to set
	 * @param type The type of the cms node 
	 *        (SetOperation.TYPE_CONTENT or SetOperation.TYPE_CONTAINER)
	 * @param holdProperties Flag to activate or deactivate the deletion of the
	 *        old node properties (if the node already exist)  
	 * @throws BuildOperationException Thrown if an error occurs during the creation
	 */
	public SetOperation(String path, String type, boolean holdProperties) throws BuildOperationException {	
		try {
			operationDescriptor = new SourceBean(Constants.NAME_SET_OPERATION);
			operationDescriptor.setAttribute(Constants.PATH, path);
			operationDescriptor.setAttribute(Constants.TYPE, type);
			SourceBean contentSB = new SourceBean(Constants.CONTENTSB);
			contentSB.setAttribute(Constants.STREAMATTR, new ByteArrayInputStream(new byte[0]));
			if(holdProperties)
				operationDescriptor.setAttribute(Constants.OLDPROPERTIES, "false");
			else 
				operationDescriptor.setAttribute(Constants.OLDPROPERTIES, "true");
			SourceBean props = new SourceBean(Constants.PROPERTIES);
			operationDescriptor.setAttribute(props);
			operationDescriptor.setAttribute(Constants.VERSIONABLE, "false");
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "SetOperation::<init>(String, String, boolean): " +
                                "Error during the creation of the set operation" +
                                "SourceBean");
			throw new BuildOperationException("SetOperation::<init>(String, String, booelan): " +
											  "Error during the creation of the " +
					                          "set operation SourceBean", e);
		}
	}
	
	
	/**
	 * Constructor of the class useful to build a set operation
	 * setting the path, type, cancelOldProperties attributes and the content.
	 * @param path The Path of the cms node to set
	 * @param type The type of the cms node 
	 *        (SetOperation.TYPE_CONTENT or SetOperation.TYPE_CONTAINER)
	 * @param holdProperties Flag to activate or deactivate the deletion of the
	 *        old node properties (if the node already exist)  
	 * @param content The content of the node. If the node is of type 
	 * 		  SetOperation.TYPE_CONTAINER the content will be ignored
	 * @throws BuildOperationException Thrown if an error occurs during the creation
	 */
	public SetOperation(String path, String type, 
			            boolean holdProperties, InputStream content) throws BuildOperationException {	
		try {
			operationDescriptor = new SourceBean(Constants.NAME_SET_OPERATION);
			operationDescriptor.setAttribute(Constants.PATH, path);
			operationDescriptor.setAttribute(Constants.TYPE, type);
			SourceBean contentSB = new SourceBean(Constants.CONTENTSB);
			contentSB.setAttribute(Constants.STREAMATTR, content);
			if(holdProperties)
				operationDescriptor.setAttribute(Constants.OLDPROPERTIES, "false");
			else 
				operationDescriptor.setAttribute(Constants.OLDPROPERTIES, "true");
			SourceBean props = new SourceBean(Constants.PROPERTIES);
			operationDescriptor.setAttribute(props);
			operationDescriptor.setAttribute(Constants.VERSIONABLE, "false");
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "SetOperation::<init>(String, String, boolean, InputStream): " +
                                "Error during the creation of the set operation" +
                                "SourceBean");
			throw new BuildOperationException("SetOperation::<init>(String, String, booelan, InputStream): " +
											  "Error during the creation of the " +
					                          "set operation SourceBean", e);
		}
	}
	
	
	/**
	 * Constructor of the class useful to build a set operation
	 * setting the path, type, cancelOldProperties attributes and a list of properties.
	 * @param path The Path of the cms node to set
	 * @param type The type of the cms node 
	 *        (SetOperation.TYPE_CONTENT or SetOperation.TYPE_CONTAINER)
	 * @param holdProperties Flag to activate or deactivate the deletion of the
	 *        old node properties (if the node already exist)  
	 * @param content The content of the node. If the node is of type 
	 * 		  SetOperation.TYPE_CONTAINER the content will be ignored
	 * @param cmsProperties The list of properties of the node. Each property of
	 *        the list must be of type it.eng.spago.cms.CmsProperty
	 * @throws BuildOperationException Thrown if an error occurs during the creation
	 */
	public SetOperation(String path, String type, boolean holdProperties, List cmsProperties) 
						throws BuildOperationException {	
		try {
			operationDescriptor = new SourceBean(Constants.NAME_SET_OPERATION);
			operationDescriptor.setAttribute(Constants.PATH, path);
			operationDescriptor.setAttribute(Constants.TYPE, type);
			SourceBean contentSB = new SourceBean(Constants.CONTENTSB);
			contentSB.setAttribute(Constants.STREAMATTR, new ByteArrayInputStream(new byte[0]));
			if(holdProperties)
				operationDescriptor.setAttribute(Constants.OLDPROPERTIES, "false");
			else 
				operationDescriptor.setAttribute(Constants.OLDPROPERTIES, "true");
			SourceBean propsSB = buildSourceBeanProperties(cmsProperties);
			operationDescriptor.setAttribute(propsSB);
			operationDescriptor.setAttribute(Constants.VERSIONABLE, "false");
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "SetOperation::<init>(String, String, boolean, List): " +
                                "Error during the creation of the set operation" +
                                "SourceBean");
			throw new BuildOperationException("SetOperation::<init>(String, String, booelan, List): " +
											  "Error during the creation of the " +
					                          "set operation SourceBean", e);
		}
	}
	
	
	
	
	/**
	 * Constructor of the class useful to build a set operation
	 * setting the path, type, cancelOldProperties attributes, 
	 * a list of properties and the content.
	 * @param path The Path of the cms node to set
	 * @param type The type of the cms node 
	 *        (SetOperation.TYPE_CONTENT or SetOperation.TYPE_CONTAINER)
	 * @param holdProperties Flag to activate or deactivate the deletion of the
	 *        old node properties (if the node already exist)  
	 * @param cmsProperties The list of properties of the node. Each property of
	 *        the list must be of type it.eng.spago.cms.CmsProperty
	 * @param content The content of the node. If the node is of type 
	 * 		  SetOperation.TYPE_CONTAINER the content will be ignored
	 * @throws BuildOperationException Thrown if an error occurs during the creation
	 */
	public SetOperation(String path, String type, boolean holdProperties, 
			            List cmsProperties, InputStream content) throws BuildOperationException {	
		try {
			operationDescriptor = new SourceBean(Constants.NAME_SET_OPERATION);
			operationDescriptor.setAttribute(Constants.PATH, path);
			operationDescriptor.setAttribute(Constants.TYPE, type);
			SourceBean contentSB = new SourceBean(Constants.CONTENTSB);
			contentSB.setAttribute(Constants.STREAMATTR, content);
			if(holdProperties)
				operationDescriptor.setAttribute(Constants.OLDPROPERTIES, "false");
			else 
				operationDescriptor.setAttribute(Constants.OLDPROPERTIES, "true");
			
			SourceBean propsSB = buildSourceBeanProperties(cmsProperties);
			operationDescriptor.setAttribute(propsSB);
			operationDescriptor.setAttribute(Constants.VERSIONABLE, "false");
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
								"SetOperation::<init>(String, String, boolean, List): " +
								"Error during the creation of the set operation" +
            					"SourceBean");
			throw new BuildOperationException("SetOperation::<init>(String, String, booelan, List): " +
						  					  "Error during the creation of the " +
						  					  "set operation SourceBean", e);
		}
	}
	
	
	/**
	 * Set the path of the cms node
	 * @param path Path of the cms node
	 * @throws BuildOperationException Thrown if an error occurs during the setting
	 */
	public void setPath(String path) throws BuildOperationException {
		try {
			operationDescriptor.updAttribute(Constants.PATH, path);
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "SetOperation::setPath: Error during the path setting");
            throw new BuildOperationException("SetOperation::setPath: Error during the path setting", e);
		}
	}
	
	/**
	 * Set the content of the cms node
	 * @param is Content InputStream of the cms node
	 * @throws BuildOperationException Thrown if an error occurs during the setting
	 */
	public void setContent(InputStream is) throws BuildOperationException {
		try {
			operationDescriptor.updAttribute(Constants.STREAM, is);
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "SetOperation::setContent: Error during the content setting");
            throw new BuildOperationException("SetOperation::setContent: Error during the content setting", e);
		}
	}
	
	/**
	 * Set the type of the cms node
 	 * @param tyoe Type of the cms node (SetOperation.TYPE_CONTENT / SetOperation.TYPE_CONTAINER)
	 * @throws BuildOperationException Thrown if an error occurs during the setting
	 */
	public void setType(String type) throws BuildOperationException {
		try {
			operationDescriptor.updAttribute(Constants.TYPE, type);
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "SetOperation::setType: Error during the type setting");
            throw new BuildOperationException("SetOperation::setType: Error during the type setting", e);
		}
	}
	
	/**
	 * Set the flag to activate or deactivate the deletion
	 * of the old cms node properties
 	 * @param erase boolean flag to activate or deactivate the deletion
	 * of the old cms node properties
	 * @throws BuildOperationException Thrown if an error occurs during the setting
	 */
	public void setEraseOldProperties(boolean erase) throws BuildOperationException {
		String eraseStr = "false";
		if(erase) {
			eraseStr = "true";
		}
		try {
			operationDescriptor.updAttribute(Constants.OLDPROPERTIES, eraseStr);
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "SetOperation::setEraseOldProperties: Error during the erase old properties setting");
            throw new BuildOperationException("SetOperation::etEraseOldProperties: Error during the erase old properties setting", e);
		}
	}
	
	
	/**
	 * Set the flag that indicates if the node (if it is a new one) must be versionable or not
 	 * @param versionable Boolean flag that indicates if the node is versionable or not
	 * @throws BuildOperationException Thrown if an error occurs during the setting
	 */
	public void setVersionable(boolean versionable) throws BuildOperationException {
		String versionableStr = "false";
		if (versionable) {
			versionableStr = "true";
		}
		try {
			operationDescriptor.updAttribute(Constants.VERSIONABLE, versionableStr);
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "SetOperation::setVersionable: Error during the versionable setting");
            throw new BuildOperationException("SetOperation::etEraseOldProperties: Error during the versionable setting", e);
		}
	}
	
	/**
	 * Set the list of properties of the node
 	 * @param properties List of tne node properties 
 	 * Each property must be of type it.eng.spago.cms.CmsProperty
	 * @throws BuildOperationException Thrown if an error occurs during the setting
	 */
	public void setProperties(List properties) throws BuildOperationException {	
		try{
			SourceBean props = buildSourceBeanProperties(properties);
			operationDescriptor.delAttribute(Constants.PROPERTIES);
			operationDescriptor.setAttribute(props);
		}catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "SetOperation::setProperties: Error during the properties setting");
            throw new BuildOperationException("SetOperation::setProperties: Error during the properties setting", e);
		}
	}
	
	/**
	 * Produce the SourceBean representation of the properties list in order
	 * to store it into the internal operation SourceBean descriptor
	 * @param properties The list of properties
	 * @return The SourceBean representation of the properties list
	 */
	private SourceBean buildSourceBeanProperties(List properties) {
		SourceBean propsSB = null;
		try {
			propsSB = new SourceBean(Constants.PROPERTIES);
			Iterator iterProps = properties.iterator();
			while(iterProps.hasNext()) {
				CmsProperty prop = (CmsProperty)iterProps.next();
				String name = prop.getName();
				String type = prop.getType();
				SourceBean propSB = new SourceBean(Constants.PROPERTY);
				propSB.setAttribute("name", name);
				propSB.setAttribute("type", type);
			    if(type.equalsIgnoreCase(CmsProperty.TYPE_STRING)) {
			    	String [] values = prop.getStringValues();
			    	for(int i=0; i<values.length; i++) {
			    		SourceBean valuePropSB = new SourceBean(Constants.VALUE_PROPERTY_SB);
			    		valuePropSB.setAttribute("value", values[i]);
			    		propSB.setAttribute(valuePropSB);
			    	}
			    }
			    if(type.equalsIgnoreCase(CmsProperty.TYPE_LONG)) {
			    	Long [] values = prop.getLongValues();
			    	for(int i=0; i<values.length; i++) {
			    		SourceBean valuePropSB = new SourceBean(Constants.VALUE_PROPERTY_SB);
			    		valuePropSB.setAttribute("value", values[i]);
			    		propSB.setAttribute(valuePropSB);
			    	}
			    }
			    if(type.equalsIgnoreCase(CmsProperty.TYPE_DOUBLE)) {
			    	Double [] values = prop.getDoubleValues();
			    	for(int i=0; i<values.length; i++) {
			    		SourceBean valuePropSB = new SourceBean(Constants.VALUE_PROPERTY_SB);
			    		valuePropSB.setAttribute("value", values[i]);
			    		propSB.setAttribute(valuePropSB);
			    	}
			    }
			    if(type.equalsIgnoreCase(CmsProperty.TYPE_DATE)) {
			    	Calendar [] values = prop.getDateValues();
			    	for(int i=0; i<values.length; i++) {
			    		SourceBean valuePropSB = new SourceBean(Constants.VALUE_PROPERTY_SB);
			    		valuePropSB.setAttribute("value", values[i]);
			    		propSB.setAttribute(valuePropSB);
			    	}
			    }
			    if(type.equalsIgnoreCase(CmsProperty.TYPE_BOOLEAN)) {
			    	Boolean [] values = prop.getBooleanValues();
			    	for(int i=0; i<values.length; i++) {
			    		SourceBean valuePropSB = new SourceBean(Constants.VALUE_PROPERTY_SB);
			    		valuePropSB.setAttribute("value", values[i]);
			    		propSB.setAttribute(valuePropSB);
			    	}
			    }
			    if(type.equalsIgnoreCase(CmsProperty.TYPE_BINARY)) {
			    	InputStream [] values = prop.getBinaryValues();
			    	for(int i=0; i<values.length; i++) {
			    		SourceBean valuePropSB = new SourceBean(Constants.VALUE_PROPERTY_SB);
			    		valuePropSB.setAttribute("value", values[i]);
			    		propSB.setAttribute(valuePropSB);
			    	}
			    }
			    // set into the properties SB the single property SB
			    propsSB.setAttribute(propSB);
			}
		} catch (Exception e) {
			return null;
		}
		return propsSB;
	}

	/**
	 * Gets the Internal SourceBean which defines the internal structure
	 * of the operation
	 * @return The Internal SourceBean of the operation which defines
	 * the internal structure of the operation
	 */
	public SourceBean getOperationDescriptor() {
		return operationDescriptor;
	}

	


	
	
}
