package it.eng.spago.cms.operations;

import it.eng.spago.base.SourceBean;
import it.eng.spago.cms.constants.Constants;
import it.eng.spago.cms.exceptions.BuildOperationException;
import it.eng.spago.tracing.TracerSingleton;

/**
 * Models a repository version operation: this means that, if the node is versionable,
 * a new version of the node is created. The operation has two attributes:
 * <ul>
 * 		<li>path: the path of the cms node to be versioned (mandatory)</li>
 * 		<li>noRecurse: flag: if it is true then only the node at path,
 * 			but not its child nodes, will be versioned; if it is false then the entire subtree rooted at
 * 			path is versioned, but only if the child nodes are versionable (optional: default is false)</li>
 * </ul>
 * The configuration of the operation is stored into a Spago SourceBean structured like below:
 * <pre>
 * &lt;OPERATION name="VERSION"&gt;
 * 	    &lt;VERSIONOPERATION path="?" /&gt;  
 * &lt;/OPERATION&gt;
 * <pre>
 */
public class VersionOperation {

	/**
	 * Internal structure of the operation
	 */
	private SourceBean operationDescriptor = null;

	/**
	 * Constructor of the class useful to build an empty version operation.
	 * @throws BuildOperationException Thrown if an error occurs during the creation
	 */
	public VersionOperation() throws BuildOperationException {
		try {
			operationDescriptor = new SourceBean(Constants.NAME_VERSION_OPERATION);
			operationDescriptor.setAttribute(Constants.PATH, "");
			operationDescriptor.setAttribute(Constants.NORECURSE, "false");
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "VersionOperation::<init>(): Error during the creation of the version operation" +
                                "SourceBean");
			throw new BuildOperationException("VersionOperation::<init>(): Error during the creation of the " +
					                          "version operation SourceBean", e);
		}
	}
	
	/**
	 * Constructor of the class useful to build a version operation setting the path.
	 * @param path The path of the cms node to be versioned
	 * @throws BuildOperationException Thrown if an error occurs during the creation
	 */
	public VersionOperation(String path) throws BuildOperationException {
		try {
			operationDescriptor = new SourceBean(Constants.NAME_VERSION_OPERATION);
			operationDescriptor.setAttribute(Constants.PATH, path);
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "VersionOperation::<init>(String path): " +
                                "Error during the creation of the version operation SourceBean");
			throw new BuildOperationException("VersionOperation::<init>(String path): " +
								"Error during the creation of the version operation SourceBean", e);
		}
	}
	
	/**
	 * Constructor of the class useful to build a version operation setting the path and the noRecurse flag.
	 * @param path The path of the cms node to be versioned
	 * @param noRecurse The noRecurse flag
	 * @throws BuildOperationException Thrown if an error occurs during the creation
	 */
	public VersionOperation(String path, boolean noRecurse) throws BuildOperationException {
		try {
			operationDescriptor = new SourceBean(Constants.NAME_VERSION_OPERATION);
			operationDescriptor.setAttribute(Constants.PATH, path);
			if (noRecurse) operationDescriptor.updAttribute(Constants.NORECURSE, "true");
			else operationDescriptor.updAttribute(Constants.NORECURSE, "false");
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "VersionOperation::<init>(String path): " +
                                "Error during the creation of the version operation SourceBean");
			throw new BuildOperationException("VersionOperation::<init>(String path): " +
								"Error during the creation of the version operation SourceBean", e);
		}
	}
	
	/**
	 * Set the path of the cms node to the versioned
	 * @param path Path of the cms node to the versioned
	 * @throws BuildOperationException Thrown if an error occurs during the setting
	 */
	public void setPath(String path) throws BuildOperationException {
		try {
			operationDescriptor.updAttribute(Constants.PATH, path);
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "VersionOperation::setPath: Error during the path setting");
            throw new BuildOperationException("VersionOperation::setPath: Error during the path setting", e);
		}
	}
	
	/**
	 * Set the noRecurse flag
	 * @param noRecurse The noRecurse flag to set
	 * @throws BuildOperationException Thrown if an error occurs during the setting
	 */
	public void setNoRecurse(boolean noRecurse) throws BuildOperationException {
		try {
			if (noRecurse) operationDescriptor.updAttribute(Constants.NORECURSE, "true");
			else operationDescriptor.updAttribute(Constants.NORECURSE, "false");
		} catch (Exception e) {
			TracerSingleton.log(Constants.NAME_MODULE, TracerSingleton.CRITICAL,
                                "VersionOperation::setContent: Error during the noRecurse flag setting");
            throw new BuildOperationException("VersionOperation::setContent: Error during the noRecurse flag " +
            					"setting", e);
		}
	}
	
	/**
	 * Gets the Internal SourceBean which defines the internal structure
	 * of the operation
	 * @return The Internal SourceBean of the operation which defines
	 * the internal structure of the operation
	 */
	public SourceBean getOperationDescriptor() {
		return operationDescriptor;
	}
	
}
