package it.eng.spago.cms.exec;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.cms.exceptions.OperationExecutionException;
import it.eng.spago.cms.exceptions.PathNotValidException;
import it.eng.spago.cms.util.OperationDescriptorUtils;
import it.eng.spago.cms.util.Path;
import it.eng.spago.cms.util.RepositoryNodeUtils;
import it.eng.spago.cms.util.constants.CMSConstants;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.tracing.TracerSingleton;

import javax.jcr.Node;
import javax.jcr.PathNotFoundException;
import javax.jcr.ReferentialIntegrityException;
import javax.jcr.RepositoryException;
import javax.jcr.Session;

/**
 * Implements method to perform a delete operation
 * into the repository
 */
public class DeleteObjectImpl {
	
	/**
	 * Execs the delete operation on the repository
	 * @param opConf, SourceBean containing the operation configuration
	 * @param conn, Connection to the repository
	 * @throws EMFInternalError thrown if an error occurs during the execution
	 */
	protected void execute(SourceBean opConf, CMSConnection conn) 
						 throws OperationExecutionException {

		Session session = conn.getSession();
		String pathStr = null;
		String versionStr = null;
		boolean versionExist = false;
		Node node = null;
		
		try {		
			// get path string new object
		    pathStr = OperationDescriptorUtils.getStringAttribute(opConf, CMSConstants.PATH);
			// get version to retrive  (version can be null)			
            versionStr = OperationDescriptorUtils.getStringAttribute(opConf, CMSConstants.VERSION);
            // create Path  
			Path path = Path.create(pathStr);
			// get node
			if(path.isRootNode()) {
				TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MAJOR,
		    	"DeleteObjectImpl::execute: cannot remove root node");				
			    throw new OperationExecutionException("DeleteObjectImpl::execute: " +
			    		  "cannot remove root node");
			} else {
		    	node = (Node)session.getRootNode().getNode(path.getRootRelativePathStr());
		    	// if the node is versionable it is checked out
		    	if(node.isNodeType("mix:versionable") && !node.isCheckedOut()) {
		    		node.checkout();
		    	}
		    }			
			// if the node is versionable and version is set controls if version exist
			if(node.isNodeType("mix:versionable") && (versionStr!=null) && !(versionStr.trim().equals("")) ) {
				versionExist = RepositoryNodeUtils.versionNameExist(node, versionStr);
				if(!versionExist) {
		    		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MAJOR,
			    	"DeleteObjectImpl::execute: the version doesn't exist");				
				    throw new OperationExecutionException("DeleteObjectImpl::execute: " +
				    		"the version doesn't exist");
		    	}
			}			
			//  if the node is versionable and the version is set then remove the version
			if (node.isNodeType("mix:versionable") && versionStr != null && !versionStr.trim().equals("")) {
				node.getVersionHistory().removeVersion(versionStr);
	    	}
			// else (the node is not versionable or the version isn't set)
	    	else {  
	    		node.remove();
	    	}
			// save changes
			session.save();	
		} catch (PathNotFoundException e) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MINOR,
	    	"DeleteObjectImpl::execute: object not found");
			throw new OperationExecutionException("DeleteObjectImpl::execute: object not found");
		} catch (ReferentialIntegrityException e) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
	    	"DeleteObjectImpl::execute: cannot erase version, there's at least one reference. Probably the version to erase is the current version of the object");
			throw new OperationExecutionException("DeleteObjectImpl::execute: cannot erase version");
		} catch (RepositoryException e) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
	    	"DeleteObjectImpl::execute: repository access error");
			throw new OperationExecutionException("DeleteObjectImpl::execute: repository access error");
		} catch (PathNotValidException e) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MAJOR,
	    	"DeleteObjectImpl::execute: malformed path");
			throw new OperationExecutionException("DeleteObjectImpl::execute: malformed path");
		} 
	}	
	
}
