package it.eng.spago.cms.exec;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.cms.CmsNode;
import it.eng.spago.cms.CmsProperty;
import it.eng.spago.cms.CmsVersion;
import it.eng.spago.cms.exceptions.OperationExecutionException;
import it.eng.spago.cms.exceptions.PathNotValidException;
import it.eng.spago.cms.util.OperationDescriptorUtils;
import it.eng.spago.cms.util.Path;
import it.eng.spago.cms.util.RepositoryNodeUtils;
import it.eng.spago.cms.util.Utils;
import it.eng.spago.cms.util.constants.OperationsConstants;
import it.eng.spago.error.EMFInternalError;
import it.eng.spago.tracing.TracerSingleton;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.PathNotFoundException;
import javax.jcr.Property;
import javax.jcr.PropertyIterator;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.Value;
import javax.jcr.version.Version;
import javax.jcr.version.VersionIterator;

/**
 * Implements method to perform a get operation
 * into the repository
 */
public class GetObjectImpl {
	
	/**
	 * Execs a get operation on a node of the repository
	 * and return an object containing all requested information.
	 * @param opConf SourceBean containing the operation descriptor
	 * @param conn Connection to the repository
	 * @return CmsNode object that models the requested node and contains
	 * all the requested information
	 * @throws EMFInternalError thrown if an error occurs during the execution
	 */
	protected CmsNode execute(SourceBean opConf, CMSConnection conn) throws OperationExecutionException {
		
		CmsNode returnNode = null;
		Session session = conn.getSession();
        String pathStr = null;  
		String verNameStr = null;
		String verLabelStr = null;
		Boolean getVersions;
		Boolean getContent;
		Boolean getChilds;
		Boolean getProperties;
		boolean versionExist;
		boolean isContainer; 
		Node node = null;
		
		try {
			// get path string new object
		    pathStr = OperationDescriptorUtils.getStringAttribute(opConf, OperationsConstants.PATH);
			// get version name to retrive  (version name can be null)			
            verNameStr = OperationDescriptorUtils.getStringAttribute(opConf, OperationsConstants.VERSION);
            // get flag of required information
            getVersions = OperationDescriptorUtils.getBooleanAttribute(opConf, OperationsConstants.GETVERSIONS);
            if(getVersions == null) { getVersions = new Boolean(false); }
            getChilds = OperationDescriptorUtils.getBooleanAttribute(opConf, OperationsConstants.GETCHILDS);
            if(getChilds == null) { getChilds = new Boolean(false); }
            getProperties = OperationDescriptorUtils.getBooleanAttribute(opConf, OperationsConstants.GETPROPERTIES);
            if(getProperties == null) { getProperties = new Boolean(false); }
            getContent = OperationDescriptorUtils.getBooleanAttribute(opConf, OperationsConstants.GETCONTENT);
            if(getContent == null) { getContent = new Boolean(false); }
            // create the path object  
			Path path = Path.create(pathStr);
			// get node
			if(path.isRootNode()) {
				node = session.getRootNode();
				isContainer = true;
			} else {
		    	node = (Node)session.getRootNode().getNode(path.getRootRelativePathStr());
			} 
			node.refresh(true);
			// if version name is set control if version exist
			if( (verNameStr!= null) && !verNameStr.trim().equalsIgnoreCase("") ) {
				versionExist = RepositoryNodeUtils.versionNameExist(node, verNameStr);
				if(!versionExist) {
		    		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MAJOR,
			    	"GetObjectImpl::execute: version doesn't exists");				
				    throw new OperationExecutionException("GetObjectImpl::execute: version doesn't exists");
		    	}
			}
            // checkout node
			if(node.isNodeType("mix:versionable")) {
				if(!node.isCheckedOut()) {
					node.checkout();
				}
			}
			returnNode = toNode(conn, node, verNameStr, getVersions.booleanValue(), 
								getChilds.booleanValue(), getProperties.booleanValue(), 
								getContent.booleanValue());
			 						
		} catch (PathNotFoundException e) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MINOR,
	    	"GetObjectImpl::execute: object not found");	
		} catch (RepositoryException e) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
	    	"GetObjectImpl::execute: repository acess error");
			throw new OperationExecutionException("GetObjectImpl::execute: repository acess error");
		} catch (PathNotValidException e) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MAJOR,
	    	"GetObjectImpl::execute: malformed path");
			throw new OperationExecutionException("GetObjectImpl::execute: malformed path");
		} 
		
		// return the result sourceBean
		return returnNode;
	}

	
	
	
	
	/**
	 * Retrives requested information from a cms node and store them 
	 * into a CmsNode object
	 * @param conn, connection to the repository
	 * @param node, node from which retrive information
	 * @param ver, name of the version to retrive
	 * @param incVer, flag to include versions information
	 * @param incChild, flag to include childs information
	 * @param incProp, flag to include properties information
	 * @param incCont, flag to include Content information
	 * @return CmsNode A CmsNode that contains all the requested information
	 * @throws EMFInternalError thrown if an error occurs during the execution
	 */
	private CmsNode toNode(CMSConnection conn, Node node, String ver, boolean incVer, 
			               boolean incChild, boolean incProp, boolean incCont) 
	                       throws OperationExecutionException {
		
		CmsNode cmsnode = null;
		String prefixSys = conn.getPrefixPropertySystem();
		String prefixUsr = conn.getPrefixPropertyUser();
		String objectType = "";
		boolean requestVer = false;
		if((ver != null) && (!ver.trim().equals("")) ) {
			requestVer = true;
		}
		Node version = null;
	  	// control if node is container or content
		boolean isContainer = true;
		try {
			Utils.debug("GetObjectImpl::toNode:start retriving type property for node :" + node.getPath());
			// root node case
			if(node.getPath().equalsIgnoreCase("/")) {
				Utils.debug("GetObjectImpl::toNode:node is the root node so it's a container for default");
				isContainer=true;
			} else {
				Property typeProp = node.getProperty( prefixSys + ":"+OperationsConstants.TYPE);
				Utils.debug("GetObjectImpl::toNode:property retrived :" + typeProp);
				try{
					Utils.debug("GetObjectImpl::toNode:start retriving values of the property (multivalue case)");
					Value[] typePropValues = typeProp.getValues();
					Value typePropVal = typePropValues[0];
					objectType = typePropVal.getString();
					Utils.debug("GetObjectImpl::toNode:value type property retrived :" + objectType);
				} catch (Exception e) {
					Utils.debug("GetObjectImpl::toNode:start retriving values of the property (single value case)");
					Value typePropVal = typeProp.getValue();
					objectType = typePropVal.getString();
					Utils.debug("GetObjectImpl::toNode:value type property retrived :" + objectType);
				}
				if( (objectType==null) || (objectType.equalsIgnoreCase(OperationsConstants.CONTAINER))) {
					isContainer = true;
					Utils.debug("GetObjectImpl::toNode:node is a container");
				} else {
					isContainer = false;
					Utils.debug("GetObjectImpl::toNode:node isn't a container, is a content");
				}
			}
		}
		catch (PathNotFoundException pnfe) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MAJOR,
								"GetObjectImpl::toSourceBean: cannot retrive property type of the node", pnfe);				
		    throw new OperationExecutionException("GetObjectImpl::execute: " +
		    					"cannot retrive property type of the node, path not found");
		}
		catch (Exception e) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MAJOR,
	    	"GetObjectImpl::toSourceBean: impossible to retrive property type of the node", e);				
		    throw new OperationExecutionException("GetObjectImpl::execute: " +
		    		"impossible to retrive property type of the node");
		}
		try {
			if(requestVer) {
				try {
					version = node.getVersionHistory().getVersion(ver).getNodes().nextNode();
				} catch (Exception e) {
					requestVer = false;
				}
			}
			String name = "";
			String uuid = "";
			String versionname = "";
			String path = "";
			String type = "";
			name = node.getName();
			path = node.getPath();
		    try{
		    	versionname = node.getBaseVersion().getName();
		    } catch(Exception e) { /* ignore */ }
			try{
				uuid = node.getUUID();
			} catch(Exception e) { /* ignore */	}
			if(isContainer) {
				type = OperationsConstants.CONTAINER;
			} else {
				type = OperationsConstants.CONTENT;
			}
			cmsnode = new CmsNode(name, path, versionname, uuid, type);
			// retrive versions
			List versions = new ArrayList();
			if(incVer && !requestVer) {
				SourceBean versionsSB = new SourceBean(OperationsConstants.VERSIONS);
				if(node.isNodeType("mix:versionable")) {
					try{
						VersionIterator vit = node.getVersionHistory().getAllVersions();
						while (vit.hasNext()){
							Version v = (Version)vit.next();
							if(!v.getName().equalsIgnoreCase("jcr:rootVersion")) {
								String namever = v.getName();
								String dataCreation = v.getCreated().getTime().toString();
								CmsVersion cmsver = new CmsVersion(namever, dataCreation);
								versions.add(cmsver);
							}
						}
					} catch (Exception e) {
						// ignore
					}
				}
				cmsnode.setVersions(versions);
			}
			// get properties
			Utils.debug("GetObjectImpl::toNode:start retriving properties\n");
			List props = new ArrayList();
			Utils.debug("GetObjectImpl::toNode:retrive properties information ? " + incProp);
			if(incProp) {
				Utils.debug("GetObjectImpl::toNode:start retriving properties\n");
				Node nodeForProp = node;
				if(requestVer){
					Utils.debug("GetObjectImpl::toNode:requested a version of the node ==> node = version \n");
					nodeForProp = version;
				}
				PropertyIterator properties = null;
				try{
					properties =  nodeForProp.getProperties(prefixUsr + ":*");
					Utils.debug("GetObjectImpl::toNode:node properties iterator retrived: " + properties );	
				} catch (Exception e) {
					Utils.debug("GetObjectImpl::toNode:error while retriving properties with use prefix");	
				}
				if(properties!=null){
					Utils.debug("GetObjectImpl::toNode:node has properties with user prefix ");		
					while (properties.hasNext()){
						CmsProperty cmsprop = null;
						Property property = (Property)properties.next();
						Utils.debug("GetObjectImpl::toNode: single property retrived: " + cmsprop );
						String nameprop = property.getName();
						Utils.debug("GetObjectImpl::toNode: single property name: " + nameprop );
						nameprop = nameprop.substring(prefixUsr.length()+1);
						Value[] values = null;
						try {
							values = property.getValues();
						} catch (Exception e) {
							values = new Value[1];
							values[0] = property.getValue();
						}
						int codType = values[0].getType();
						Utils.debug("GetObjectImpl::toNode: single property code type: " + codType );
						String proptype = PropertyType.nameFromValue(codType);	
						Utils.debug("GetObjectImpl::toNode: single property type: " + proptype );
						int numValues = values.length;
						if(proptype.equalsIgnoreCase(CmsProperty.TYPE_STRING)) {
							String[] propvalues = new String[numValues];
							for(int i=0; i<values.length; i++)
								propvalues[i] = values[i].getString();
							cmsprop = new CmsProperty(nameprop, propvalues);
						}
						if(proptype.equalsIgnoreCase(CmsProperty.TYPE_LONG)) {
							Long[] propvalues = new Long[numValues];
							for(int i=0; i<values.length; i++)
								propvalues[i] = new Long(values[i].getLong());
							cmsprop = new CmsProperty(nameprop, propvalues);
						}
						if(proptype.equalsIgnoreCase(CmsProperty.TYPE_DOUBLE)) {
							Double[] propvalues = new Double[numValues];
							for(int i=0; i<values.length; i++)
								propvalues[i] = new Double(values[i].getString());
							cmsprop = new CmsProperty(nameprop, propvalues);
						}
						if(proptype.equalsIgnoreCase(CmsProperty.TYPE_DATE)) {
							Calendar[] propvalues = new Calendar[numValues];
							for(int i=0; i<values.length; i++)
								propvalues[i] = values[i].getDate();
							cmsprop = new CmsProperty(nameprop, propvalues);
						}
						if(proptype.equalsIgnoreCase(CmsProperty.TYPE_BOOLEAN)) {
							Boolean[] propvalues = new Boolean[numValues];
							for(int i=0; i<values.length; i++)
								propvalues[i] = new Boolean(values[i].getBoolean());
							cmsprop = new CmsProperty(nameprop, propvalues);
						}
						if(proptype.equalsIgnoreCase(CmsProperty.TYPE_BINARY)) {
							InputStream[] propvalues = new InputStream[numValues];
							for(int i=0; i<values.length; i++)
								propvalues[i] = values[i].getStream();
							cmsprop = new CmsProperty(nameprop, propvalues);
						}
						props.add(cmsprop);
					}
					Utils.debug("GetObjectImpl::toNode:node properties list retrived: " + props + "\n");
				} else {
					Utils.debug("GetObjectImpl::toNode:node doens't have properties with user prefix ");		
				}
			} 
			cmsnode.setProperties(props);
			Utils.debug("GetObjectImpl::toNode:node properties stored into returned object: " + cmsnode.getProperties() );
			
			// getChilds
			List childList = new ArrayList();
			Utils.debug("GetObjectImpl::toNode:retrive child information ? " + incChild);
			Utils.debug("GetObjectImpl::toNode:node is a container ? " + isContainer);
			if(incChild && isContainer) {
				Utils.debug("GetObjectImpl::toNode:start retriving childs\n");
				Node nodeForChild = node;
				if(requestVer){
					Utils.debug("GetObjectImpl::toNode:requested a version of the node ==> node = version \n");
					nodeForChild = version;
				}
				NodeIterator childs = nodeForChild.getNodes();
				Utils.debug("GetObjectImpl::toNode:node child iterator retrived: " + childs + "\n" );
				while (childs.hasNext()){
					CmsNode cmsChildNode = null;
					String nameChildNode = "";
					String pathChild = "";
					String uuidChild = "";
					String objectTypeChild = "";
					Node child = null;
					try {
						child = (Node)childs.next();
					} catch (Exception e) {
						continue;
					}
					Utils.debug("GetObjectImpl::toNode:single child node obtained: " + child + "\n" );
					// if node starts with jcr prefix is a system node 
					nameChildNode = child.getName();
					Utils.debug("GetObjectImpl::toNode:single child node name: " + nameChildNode + "\n" );
					if(nameChildNode.startsWith("jcr:")) {
						continue;
					}
					if(!requestVer) {
						pathChild = child.getPath();
					}
					try {
						uuidChild = child.getUUID();
					} catch (Exception e) { /* ignore */ }
					try{
						Utils.debug("GetObjectImpl::toNode:start retriving child node type property \n" );
						Property childTypeProp = child.getProperty(prefixSys + ":" + OperationsConstants.TYPE);
						Utils.debug("GetObjectImpl::toNode:start retriving child node type property \n" );
						try{
							Utils.debug("GetObjectImpl::toNode:retriving child node value of type property (multivalue case) \n" );
							Value[] childTypePropVals = childTypeProp.getValues();
							Value childTypePropVal = childTypePropVals[0];
							objectTypeChild = childTypePropVal.getString();
							Utils.debug("GetObjectImpl::toNode:child node type property value: " + objectTypeChild   +" \n" );
						} catch (Exception e) {
							Utils.debug("GetObjectImpl::toNode:retriving child node value of type property (single value case)\n" );
							Value childTypePropVal = childTypeProp.getValue();
							objectTypeChild = childTypePropVal.getString();
							Utils.debug("GetObjectImpl::toNode:child node type property value: " + objectTypeChild   +" \n" );
						}
					} catch (Exception e) { 
						/* ignore */ 
						Utils.debug("GetObjectImpl::toNode:error while retriving child node type property \n" );
					}
					cmsChildNode = new CmsNode(nameChildNode, pathChild, "", uuidChild, objectTypeChild);
					childList.add(cmsChildNode);
				}
				Utils.debug("GetObjectImpl::toNode:node childs retrived: " + childs );
			}
			cmsnode.setChilds(childList);
			Utils.debug("GetObjectImpl::toNode:node childs stored into returned object: " + cmsnode.getChilds() );
		    
		    // get content
			Utils.debug("GetObjectImpl::toNode:retrive content information ? " + incCont);
			Utils.debug("GetObjectImpl::toNode:node is a container ? " + isContainer);
            if(incCont && !isContainer) { 
            	Utils.debug("GetObjectImpl::toNode:start retriving content\n");
            	Node nodeForCont = node;
				if(requestVer){
					Utils.debug("GetObjectImpl::toNode:requested a version of the node ==> node = version \n");
					nodeForCont = version;
				}
				try {
					Property propjcr = nodeForCont.getProperty(prefixSys + ":" + OperationsConstants.STREAM);
					InputStream stream = null;  
					try{
						Value[] valuesjcr = propjcr.getValues();
						Value contval = valuesjcr[0];
						stream = contval.getStream();
					} catch (Exception e) {
						stream = propjcr.getStream(); 
					}
					cmsnode.setContent(stream);
				} catch (Exception e) {
					cmsnode.setContent(null);
				}
			}
            Utils.debug("GetObjectImpl::toNode:node content stored into returned object: " + cmsnode.getContent() ); 
		}
		catch (SourceBeanException e){
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MAJOR,
	    	"GetObjectImpl::toSourceBean: sourceBean error ");
			throw new OperationExecutionException("GetObjectImpl::toSourceBean: sourceBean error", e);
		} catch (PathNotFoundException e) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MINOR,
	    	"GetObjectImpl::toSourceBean: path not found");
			throw new OperationExecutionException("GetObjectImpl::toSourceBean: path not find", e);
		} catch (RepositoryException e) {
			TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
	    	"GetObjectImpl::toSourceBean: repository access error");
			throw new OperationExecutionException("GetObjectImpl::toSourceBean: repository access error", e);
		}
		
		return cmsnode;
	}
	
	
}
	
	

	
	
