package it.eng.spago.cms.repositoryproviders;

import it.eng.spago.base.SourceBean;
import it.eng.spago.cms.IRepositoryProvider;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;

import javax.jcr.Repository;

import org.apache.jackrabbit.core.RepositoryImpl;
import org.apache.jackrabbit.core.config.RepositoryConfig;
import org.xml.sax.InputSource;

/**
 * Implements methods to get the JSR 170 Repository instance using the jackrabbit library.
 */
public class JackrabbitRepositoryProvider implements IRepositoryProvider {

	/**
	 * Name of the module
	 */
	private static final String NAME_MODULE = "SPAGOCMS";
	
	/**
	 * JSR 170 Repository instance
	 */
	protected Repository _repo = null;

	
	/**
	 * Construct of the class, gets the JSR 170 repository object instance using the jackrabbit library.
	 * It needs two parameters 
	 * <ul>
	 * 		<li>repository path: the file system folder path where the cms will store its contents</li>
	 * 		<li>configuration file path: path to the jackrabbit configuration file</li>
	 * </ul> 
	 * into the Spago configuration files must be present an xml envelope structured like below:
	 * <pre>
	 * &lt;CONTENTCONFIGURATION&gt;
	 * 		&lt;CONTENTREPOSITORY&gt;
	 * 			&lt;PARAMETERS&gt;
	 * 					&lt;PARAMETER name="repository_path" value="file system root of the cms repository" /&gt;
	 * 					&lt;PARAMETER name="conf_file_path" value="path of the jackrabbit configuration file" /&gt;
	 * 			&lt;/PARAMETERS&gt;
	 * </pre>
	 */
	public JackrabbitRepositoryProvider() {
		try{	
		    // get spago configuration
			ConfigSingleton config = ConfigSingleton.getInstance();
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"JackrabbitRepositoryProvider::init:Spago configuration retrived");
			// try to get the parameters sourcebean from the cms configuration file
			String nameParamsSB = "CONTENTCONFIGURATION.CONTENTREPOSITORY.PARAMETERS";
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
					"JackrabbitRepositoryProvider::init:try to get the "+nameParamsSB+
					" parameter from the cms configuration file");
			SourceBean paramsSB = (SourceBean)config.getAttribute(nameParamsSB);
			if(paramsSB==null) {
				TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
						"JackrabbitRepositoryProvider::init: parameters sourcebean " +
						"not found in cms configuration");
				return;
			} else {
				TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
					"JackrabbitRepositoryProvider::init: parameters sourcebean retrived " +
					"from cms configuration " + paramsSB.toXML(false));
			} 
			// try to get the repository path parameter
			SourceBean repPathSB = (SourceBean)paramsSB.getFilteredSourceBeanAttribute("PARAMETER", "name", "repository_path");
			if(repPathSB==null) {
				TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
						"JackrabbitRepositoryProvider::init: repository path parameter " +
						"not found in cms configuration");
				return;
			} 
			String pathRepository = (String)repPathSB.getAttribute("value");
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
					"JackrabbitRepositoryProvider::init: repository path parameter retrived: " + pathRepository);
			// try to get the configuration file path parameter
			SourceBean confPathSB = (SourceBean)paramsSB.getFilteredSourceBeanAttribute("PARAMETER", "name", "conf_file_path");
			if(confPathSB==null) {
				TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
						"JackrabbitRepositoryProvider::init: configuration file path parameter " +
						"not found in cms configuration");
				return;
			} 
			String pathConfFile = (String)confPathSB.getAttribute("value");
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
					"JackrabbitRepositoryProvider::init: configuration file path parameter retrived: " + pathConfFile);
			// resolve relative paths
			if( !pathConfFile.startsWith("/") && (pathConfFile.charAt(1)!=':') ) {
				pathConfFile = ConfigSingleton.getRootPath() + "/" + pathConfFile;
				TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
									"JackrabbitRepositoryProvider::init: relative " +
									"path configuration file found, resolved to : " + pathConfFile);
			}
			if( !pathRepository.startsWith("/") && (pathRepository.charAt(1)!=':') ) {
				pathRepository = ConfigSingleton.getRootPath() + "/" + pathRepository;
				TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
						"JackrabbitRepositoryProvider::init: relative " +
						"path repository found, resolved to : " + pathRepository);
			}
			// delete dot folder path ("/./") from paths
			String dotFolderPath = File.separator + "." + File.separator;
			while(pathConfFile.indexOf(dotFolderPath) != -1) {
				int startInd = pathConfFile.indexOf(dotFolderPath);
				int endInd = startInd + 2;
				pathConfFile = pathConfFile.substring(0, startInd) + pathConfFile.substring(endInd);
				TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
									"JackrabbitRepositoryProvider::init: dot folder path " +
									"removed from path of the configuration file, new path : " + pathConfFile);
			}
			while(pathRepository.indexOf(dotFolderPath) != -1) {
				int startInd = pathRepository.indexOf(dotFolderPath);
				int endInd = startInd + 2;
				pathRepository = pathRepository.substring(0, startInd) + pathRepository.substring(endInd);
				TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
									"JackrabbitRepositoryProvider::init: dot folder path " +
									"removed from path of teh repository, new path : " + pathRepository);
			}
			// get the input stream of the configuration file
			InputStream configFileIS = null;
			try{
				configFileIS = new FileInputStream(pathConfFile);
			} catch (Exception e) {
				TracerSingleton.log(NAME_MODULE, TracerSingleton.CRITICAL,
									"JackRabbitContentRepository::init: jackrabbit configuration file " +
									"reading error", e);
			}
		    InputSource repFileConf = new InputSource(configFileIS);
		    // create repository
		    try{
		    	RepositoryConfig repConf = RepositoryConfig.create(repFileConf, pathRepository);
		    	_repo = RepositoryImpl.create(repConf);
		    } catch (Exception e) {
		    	TracerSingleton.log(NAME_MODULE, TracerSingleton.CRITICAL,
						"JackRabbitContentRepository::init: jackrabbit repository creation error", e);
		    }
		    // close input stream configuration file
		    configFileIS.close();
		} catch (Exception e) {
			TracerSingleton.log(NAME_MODULE, TracerSingleton.CRITICAL,
					            "JackRabbitContentRepository::init: error during " +
					            "the creation of the jackrabbit repository", e);
		}
	}
	

	/**
	 * Gets the JSR 170 repository object instance
	 * @param Repository the JSR 170 repository object instance 
	 */
	public Repository getRepository() {
		return _repo;
	}
	
}





