package it.eng.spago.cms.repositoryproviders;

import it.eng.spago.base.SourceBean;
import it.eng.spago.cms.IRepositoryProvider;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;

import javax.jcr.Repository;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

/**
 * Implements methods to get the JSR 170 Repository instance from a jndi context
 */
public class JndiRepositoryProvider implements IRepositoryProvider {

	/**
	 * Name of the module
	 */
	private static final String NAME_MODULE = "SPAGOCMS";
	
	/**
	 * JSR 170 Repository instance
	 */
	protected Repository _repo = null;

	
	/**
	 * Construct of the class, gets the JSR 170 repository object instance from a jndi context.
	 * Obviously the JSR 170 repository must have been already registered into the jndi context
	 * It needs two parameters: 
	 * <ul>
	 * 		<li>jndi context name: the name of the jndi context that contains the jndi resource</li>
	 * 		<li>jndi object name: the jndi name of the jndi resource</li>
	 * </ul> 
	 * into the Spago configuration files must be present an xml envelope structured like below:
	 * <pre>
	 * &lt;CONTENTCONFIGURATION&gt;
	 * 	&lt;CONTENTREPOSITORY&gt;
	 * 		&lt;PARAMETERS&gt;
	 * 			&lt;JNDICONTEXTNAME name="name of the jndi context" /&gt;
	 * 			&lt;JNDIOBJECTNAME name="name of the jndi jsr 170 repository resource" /&gt;
	 * 		&lt;/PARAMETERS&gt;
	 * </pre>
	 */
	public JndiRepositoryProvider() {
		try{	
			String paramContext = "CONTENTCONFIGURATION.CONTENTREPOSITORY.PARAMETERS.JNDICONTEXTNAME";
			String paramObject = "CONTENTCONFIGURATION.CONTENTREPOSITORY.PARAMETERS.JNDIOBJECTNAME";
			ConfigSingleton config = ConfigSingleton.getInstance();
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"JndiRepositoryProvider::init:Spago configuration retrived: " + config);
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"JndiRepositoryProvider::init:try to get the "+paramContext+
								" parameter from the cms configuration ");
			SourceBean paramContextSB = (SourceBean)config.getAttribute(paramContext);
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"JndiRepositoryProvider::init:cms sourcebean parameter jndi context " +
								"name retrived: " + paramContextSB);
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
					            "JndiRepositoryProvider::init:try to get the "+paramObject+
					            " parameter from the cms configuration ");
			SourceBean paramObjectSB = (SourceBean)config.getAttribute(paramObject);
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"JndiRepositoryProvider::init:cms sourcebean parameter jndi object " +
								"name retrived: " + paramObjectSB);
			String nameContext = (String)paramContextSB.getAttribute("name");
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"JndiRepositoryProvider::init:use "+nameContext+" context name");
			String nameObject = (String)paramObjectSB.getAttribute("name");
			TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"JndiRepositoryProvider::init:use "+nameObject+" object name");
            InitialContext initCtx = new InitialContext();
            TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
					            "JndiRepositoryProvider::init:jndi initial context created: " + initCtx);
            Context envCtx = (Context)initCtx.lookup(nameContext);
            TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
            					"JndiRepositoryProvider::init:jndi context "+nameContext+" retrived: " + envCtx);
            Object obj = envCtx.lookup(nameObject);
            TracerSingleton.log(NAME_MODULE, TracerSingleton.DEBUG, 
								"JndiRepositoryProvider::init:jndi object "+nameObject+" retrived: " + obj);
	        _repo = (Repository)obj;
	        
		} catch (NamingException ne) {
			TracerSingleton.log(NAME_MODULE, 
        			TracerSingleton.CRITICAL, 
        			"JndiRepositoryProvider::init: cannot get the repository", ne); 
		} catch (Exception e) { 
			TracerSingleton.log(NAME_MODULE, 
		            			TracerSingleton.CRITICAL, 
		            			"JndiRepositoryProvider::init: cannot get the repository", e); 
		} finally {	}
	}
	
	/**
	 * Gets the JSR 170 repository object instance
	 * @param Repository the JSR 170 repository object instance 
	 */
	public Repository getRepository() {
		return _repo;
	}


}




